import time, numpy as n, gmane as g
from scipy import stats as st
from .tableHelpers import lTable
TT=time.time()
def dl(fname,hl,vl,over=0):
    fname+=".tex"
    foo=g.doubleLines(fname,hlines=hl,vlines=vl)
    if not over:
        tablefname=fname.replace(".tex","_.tex")
    g.writeTex(foo,tablefname)
def me(fname,mark,locs,over=1):
    fn=fname+".tex"
    foo=g.markEntries_(fn,mark,locs)
    if not over:
        tn=tn.replace(".tex","_.tex")
    g.writeTex(foo,fn)

def kolmogorovSmirnovDistance(seq1,seq2,bins=300):
    """Calculate distance between histograms
    
    Adapted from the Kolmogorov-Smirnov test"""
    amin=min(min(seq1),min(seq2))
    amax=max(max(seq1),max(seq2))
    bins=n.linspace(amin,amax,bins+1,endpoint=True)
    h1=n.histogram(seq1,bins,density=True)[0]
    h2=n.histogram(seq2,bins,density=True)[0]
    space=bins[1]-bins[0]
    cs1=n.cumsum(h1*space)
    cs2=n.cumsum(h2*space)

    dc=n.abs(cs1-cs2)
    Dnn=max(dc)
    n1=len(seq1)
    n2=len(seq2)
    fact=((n1+n2)/(n1*n2))**0.5
    calpha=Dnn/fact
    return calpha
def kolmogorovSmirnovDistance_(seq1,seq2,bins=300):
    """Calculate distance between histograms
    
    Adapted from the Kolmogorov-Smirnov test,
    returns calpha, fact and Dnn"""
    amin=min(min(seq1),min(seq2))
    amax=max(max(seq1),max(seq2))
    bins=n.linspace(amin,amax,bins+1,endpoint=True)
    h1=n.histogram(seq1,bins,density=True)[0]
    h2=n.histogram(seq2,bins,density=True)[0]
    space=bins[1]-bins[0]
    cs1=n.cumsum(h1*space)
    cs2=n.cumsum(h2*space)

    dc=n.abs(cs1-cs2)
    Dnn=max(dc)
    n1=len(seq1)
    n2=len(seq2)
    fact=((n1*n2)/(n1+n2))**0.5
    calpha=Dnn*fact
    return calpha, fact, Dnn
def kolmogorovSmirnovDistance__(seq1,seq2,bins=300):
    """Calculate distance between histograms
    
    Adapted from the Kolmogorov-Smirnov test,
    returns calpha, and Dnn"""
    amin=min(min(seq1),min(seq2))
    amax=max(max(seq1),max(seq2))
    bins=n.linspace(amin,amax,bins+1,endpoint=True)
    h1=n.histogram(seq1,bins,density=True)[0]
    h2=n.histogram(seq2,bins,density=True)[0]
    space=bins[1]-bins[0]
    cs1=n.cumsum(h1*space)
    cs2=n.cumsum(h2*space)

    dc=n.abs(cs1-cs2)
    Dnn=max(dc)
    n1=len(seq1)
    n2=len(seq2)
    fact=((n1*n2)/(n1+n2))**0.5
    calpha=Dnn*fact
    return calpha, Dnn

def dnnNorm(m1,d1,m2,d2,lb=-4,rb=4,NE=1000000):
    a=st.norm(m1,d1)
    b=st.norm(m2,d2)
    domain=n.linspace(lb,rb,NE)
    avals=a.cdf(domain)
    bvals=b.cdf(domain)
    diffN=n.abs(avals-bvals).max()
    return diffN
def dnnUni(lb,rb,lb2,rb2,lbd=-1,rbd=4,NE=1000000):
    rb_=rb-lb
    rb2_=rb2-lb2
    a=st.uniform(lb,rb_)
    b=st.uniform(lb2,rb2_)
    domain=n.linspace(lbd,rbd,NE)
    avals=a.cdf(domain)
    bvals=b.cdf(domain)
    diffU=n.abs(avals-bvals).max()
    return diffU

def weib(x,nn,a):
    return (a / nn) * (x / nn)**(a - 1) * n.exp(-(x / nn)**a)
def dnnWeib(shape1,shape2,lb=0.00001,rb=10,NE=1000000):
    x=n.linspace(lb,rb,NE)
    step=x[1]-x[0]
    W=weib(x, 1., shape1)
    #W_=W/((W*step).sum())
    W_=W/((W).sum())
    W__=n.cumsum(W_)
    W2=weib(x, 1., shape2)
    #W2_=W2/((W2*step).sum())
    W2_=W2/((W2).sum())
    W2__=n.cumsum(W2_)
    diffW=n.abs(W__-W2__).max()
    return diffW
def dnnPower(shape1,shape2,lb=0,rb=1,NE=1000000):
    a=st.powerlaw(shape1)
    b=st.powerlaw(shape2)
    domain=n.linspace(lb,rb,NE)
    avals=a.cdf(domain)
    bvals=b.cdf(domain)
    diffP=n.abs(avals-bvals).max()
    return diffP





def min3(narray):
    narray_=n.array(narray)
    args=narray_.argsort()
    return narray_[args[:3]]
def max3(narray):
    narray_=n.array(narray)
    args=narray_.argsort()
    return narray_[args[-3:]]
def check(amsg="string message"):
    global TT
    print(amsg, time.time()-TT); TT=time.time()
class KSReferences:
    """Hold references and simulation routines"""
    alphas=[0.1,0.05,0.025,0.01,0.005,0.001]
    calphas=[1.22,1.36,1.48,1.63,1.73,1.95]
    def __init__(self,NC=100,NE=1000,NB=300,adir="/home/r/repos/kolmogorov-smirnov/",NE2=1000,make_all=True):
        table_dir=adir+"tables/"
        aux_dir=adir+"aux/"
        if not NE2:
            NE2=NE
        if make_all:
            self.makeAllTables(NC,NE,NE2,NB,aux_dir,table_dir)
        mvars=("NC","NE","NE2","NB","adir")
        self.vdict={}
        for mvar in mvars:
            self.vdict[mvar] = locals()[mvar]
    def makeAllTables(self,NC,NE,NE2,NB,aux_dir,table_dir):
        #self.makePreambule(NC,NE,NE2,NB,aux_dir)
        #self.makeNormalVerification(NC,NE,NE2,NB,table_dir)
        #self.makeUniformVerification(NC,NE,NE2,NB,table_dir)
        #self.makeWeibullVerification(NC,NE,NE2,NB,table_dir)
        #self.makePowerVerification(NC,NE,NE2,NB,table_dir)
        self.makeNormalDifferencesDispersion(NC,NE,NE2,NB,table_dir)
        self.makeNormalDifferencesMean(NC,NE,NE2,NB,table_dir)
        self.makeUniformDifferencesDispersion(NC,NE,NE2,NB,table_dir)
        self.makeUniformDifferencesMean(NC,NE,NE2,NB,table_dir)
        self.makeWeibullDifferencesShape(NC,NE,NE2,NB,table_dir)
        self.makePowerDifferencesShape(NC,NE,NE2,NB,table_dir)
        self.makeNormalDifferencesSamples(NC,NB,table_dir)
        self.makeNormalDifferencesSamples2(NC,NB,table_dir)
        self.makeUniformDifferencesSamples(NC,NB,table_dir)
        self.makeUniformDifferencesSamples2(NC,NB,table_dir)
        self.makeWeibullDifferencesSamples(NC,NB,table_dir)
        self.makePowerDifferencesSamples(NC,NB,table_dir)
        self.enhanceTables(table_dir)

    def makePowerDifferencesSamples(self,NC,NB,table_dir):
        #xx=n.arange(.7,2.7,0.2)
        xx=n.logspace(2,5,4)
        labels=xx
        shapes=(1.5,1.7)
        distsAllW=[[kolmogorovSmirnovDistance_(
                n.random.power(shapes[0],xxx),
                n.random.power(shapes[1],xxx), NB) for i in range(NC)]
                for xxx in xx]
        distsAllWC=[[i[0] for i in j] for j in distsAllW]
        dnns=[[i[2] for i in j] for j in distsAllW]
        data=[(n.mean(dd),n.std(dd),n.median(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllWC]
        data_=[]
        i=0
        for dists in distsAllWC:
            line=[]
            dnn=dnns[i]
            kline=[n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        diffP=dnnPower(shapes[0],shapes[1])
        caption=r"""Measurements of $c$ through simulations
        with fixed power distributions but different number of samples.
        One distribution has shape parameter a={}.
        The other distribution has a={}.
        The KS statistic of these distributions converges
        to {:.4f} as sample sizes increases.""".format(shapes[0],shapes[1],diffP)
        labelsh=[r"$n=n'$",r"$\mu(c)$",r"$\sigma(c)$","m(c)","min(c)","max(c)",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabPowerDiffSamples.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmSamp_")
        i=0
        check("table {} written at {}".format(fname,table_dir))
    def makeWeibullDifferencesSamples(self,NC,NB,table_dir):
        #xx=n.arange(.7,2.7,0.2)
        xx=n.logspace(2,5,4)
        labels=xx
        shapes=(1.5,1.7)
        distsAllW=[[kolmogorovSmirnovDistance_(
                n.random.weibull(shapes[0],xxx),
                n.random.weibull(shapes[1],xxx), NB) for i in range(NC)]
                for xxx in xx]
        distsAllWC=[[i[0] for i in j] for j in distsAllW]
        dnns=[[i[2] for i in j] for j in distsAllW]
        data=[(n.mean(dd),n.std(dd),n.median(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllWC]
        data_=[]
        i=0
        for dists in distsAllWC:
            line=[]
            dnn=dnns[i]
            kline=[n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        #x=n.linspace(0,20,100000)
        #step=x[1]-x[0]
        #W=weib(x, 1., 1.5)
        #W_=W/(W*step).sum()
        #W2=weib(x, 1., 1.7)
        #W2_=W2/(W2*step).sum()
        #diffW=n.abs(W_-W2_).max()
        diffW=dnnWeib(shapes[0],shapes[1])
        caption=r"""Measurements of $c$ through simulations
        with fixed Weibull distributions but different number of samples.
        One distribution has shape parameter $a={}$.
        The other distribution has $a={}$.
        The KS statistic of these distributions converges
        to {:.4f} as sample sizes increases.""".format(shapes[0],shapes[1],diffW)
        labelsh=[r"$n=n'$",r"$\mu(c)$",r"$\sigma(c)$","m(c)","min(c)","max(c)",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabWeibullDiffSamples.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmSamp_")
        i=0
        check("table {} written at {}".format(fname,table_dir))
    def makeUniformDifferencesSamples2(self,NC,NB,table_dir):
        #xx=n.arange(.7,2.7,0.2)
        xx=n.logspace(2,5,4)
        labels=xx
        distsAllW=[[kolmogorovSmirnovDistance_(
                n.random.random(xxx)*1.2-0.1,
                n.random.random(xxx), NB) for i in range(NC)]
                for xxx in xx]
        distsAllWC=[[i[0] for i in j] for j in distsAllW]
        dnns=[[i[2] for i in j] for j in distsAllW]
        data=[(n.mean(dd),n.std(dd),n.median(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllWC]
        data_=[]
        i=0
        for dists in distsAllWC:
            line=[]
            dnn=dnns[i]
            kline=[n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        diffU2=dnnUni(0,1,-.1,1.1)
        caption=r"""Measurements of $c$ through simulations
        with fixed uniform distributions but different number of samples.
        One distribution is uniform in [0,1].
        The other distribution is uniform in [-0.1,1.1].
        The KS statistic of these distributions converges
        to {:.4f} as sample sizes increases.""".format(diffU2)
        labelsh=[r"$n=n'$",r"$\mu(c)$",r"$\sigma(c)$","m(c)","min(c)","max(c)",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabUniformDiffSamples2.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmSamp_")
        i=0
        check("table {} written at {}".format(fname,table_dir))


    def makeUniformDifferencesSamples(self,NC,NB,table_dir):
        #xx=n.arange(.7,2.7,0.2)
        xx=n.logspace(2,5,4)
        labels=xx
        distsAllW=[[kolmogorovSmirnovDistance_(
                n.random.random(xxx)+0.05,n.random.random(xxx),NB) for i in range(NC)]

                for xxx in xx]
        distsAllWC=[[i[0] for i in j] for j in distsAllW]
        dnns=[[i[2] for i in j] for j in distsAllW]
        data=[(n.mean(dd),n.std(dd),n.median(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllWC]
        data_=[]
        i=0
        for dists in distsAllWC:
            line=[]
            dnn=dnns[i]
            kline=[n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        diffU=dnnUni(0,1,0.05,1.05)
        caption=r"""Measurements of $c$ through simulations
        with fixed uniform distributions but different number of samples.
        One distribution is uniform in [0,1].
        The other distribution is uniform in [0.05,1.05].
        The KS statistic of these distributions converges
        to {:.4f} as sample sizes increases.""".format(diffU)
        labelsh=[r"$n=n'$",r"$\mu(c)$",r"$\sigma(c)$","m(c)","min(c)","max(c)",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabUniformDiffSamples.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmSamp_")
        i=0
        check("table {} written at {}".format(fname,table_dir))



    def makeNormalDifferencesSamples2(self,NC,NB,table_dir):
        #xx=n.arange(.7,2.7,0.2)
        xx=n.logspace(2,5,4)
        labels=xx
        shape=(0,1.2)
        distsAllW=[[kolmogorovSmirnovDistance_(
                n.random.normal(0,1,xxx),
                    n.random.normal(shape[0],shape[1],xxx),NB) for i in range(NC)]

                for xxx in xx]
        distsAllWC=[[i[0] for i in j] for j in distsAllW]
        dnns=[[i[2] for i in j] for j in distsAllW]
        data=[(n.mean(dd),n.std(dd),n.median(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllWC]
        data_=[]
        i=0
        for dists in distsAllWC:
            line=[]
            dnn=dnns[i]
            kline=[n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        diffN2=dnnNorm(0,1,shape[0],shape[1])
        caption=r"""Measurements of $c$ through simulations
        with fixed normal distributions but different number of samples.
        One normal distribution has $\mu=0$ and $\sigma=1$.
        The other normal distribution have
        $\mu={}$ and $\sigma={}$.
        The KS statistic of these distributions converges
        to {:.4f} as sample sizes increases.""".format(shape[0],shape[1],diffN2)
        labelsh=[r"$n=n'$",r"$\mu(c)$",r"$\sigma(c)$","$m(c)$","$min(c)$","$max(c)$",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabNormalDiffSamples2.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmSamp_")
        i=0
        check("table {} written at {}".format(fname,table_dir))

    def makeNormalDifferencesSamples(self,NC,NB,table_dir):
        #xx=n.arange(.7,2.7,0.2)
        xx=n.logspace(2,5,4)
        labels=xx
        distsAllW=[[kolmogorovSmirnovDistance_(
                n.random.normal(0,1,xxx),
                n.random.normal(0.1,1,xxx), NB) for i in range(NC)]

                for xxx in xx]
        distsAllWC=[[i[0] for i in j] for j in distsAllW]
        dnns=[[i[2] for i in j] for j in distsAllW]
        data=[(n.mean(dd),n.std(dd),n.median(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllWC]
        data_=[]
        i=0
        for dists in distsAllWC:
            line=[]
            dnn=dnns[i]
            kline=[n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        diffN=dnnNorm(0,1,0.1,1)
        caption=r"""Measurements of $c$ through simulations
        with fixed normal distributions but different number of samples.
        One normal distribution has $\mu=0$ and $\sigma=1$.
        The other normal distribution have $\mu=0.1$ and $\sigma=1$.
        The KS statistic of these distributions converges
        to {:.4f} as sample sizes increases.""".format(diffN)
        labelsh=[r"$n=n'$",r"$\mu(c)$",r"$\sigma(c)$",r"$m(c)$",r"$min(c)$",r"$max(c)$",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabNormalDiffSamples.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmSamp_")
        i=0
        check("table {} written at {}".format(fname,table_dir))
    def enhanceTables(self,table_dir):
        dl(table_dir+"tabNormNull",[1],[])
        me(table_dir+"tabNormNull_","\\bf",[(i,1) for i in      range(1,7)])
        dl(table_dir+"tabUniformNull",[1],[])
        me(table_dir+"tabUniformNull_","\\bf",[(i,1) for i in      range(1,7)])
        dl(table_dir+"tabWeibullNull",[1],[])
        me(table_dir+"tabWeibullNull_","\\bf",[(i,1) for i in      range(1,7)])
        dl(table_dir+"tabPowerNull",[1],[])
        me(table_dir+"tabPowerNull_","\\bf",[(i,1) for i in      range(1,7)])
        dl(table_dir+"tabNormDiff3",[1],[],)
        me(table_dir+"tabNormDiff3_","\\bf",[(6,i) for i in         range(0,12)])
        dl(table_dir+"tabNormDiffMean",[1],[],)
        me(table_dir+"tabNormDiffMean_","\\bf",[(1,i) for i in      range(0,14)])
        g.fSize(table_dir+"tabNormDiffMean_.tex",r"\scriptsize",1)

        dl(table_dir+"tabNormDiffDispersion",[1],[],)
        me(table_dir+"tabNormDiffDispersion_","\\bf",[(6,i) for i in      range(0,14)])
        g.fSize(table_dir+"tabNormDiffDispersion_.tex",r"\scriptsize",1)

        dl(table_dir+"tabUniformDiffSpread",[1],[],)
        me(table_dir+"tabUniformDiffSpread_","\\bf",[(7,i) for i in range(0,14)])
        g.fSize(table_dir+"tabUniformDiffSpread_.tex",r"\scriptsize",1)
        dl(table_dir+"tabUniformDiffMean",[1],[],)
        me(table_dir+"tabUniformDiffMean_","\\bf",[(1,i) for i in   range(0,14)])
        g.fSize(table_dir+"tabUniformDiffMean_.tex",r"\scriptsize",1)
        dl(table_dir+"tabWeibullDiffShape",[1],[],)
        me(table_dir+"tabWeibullDiffShape_","\\bf",[(5,i) for i in  range(0,14)])
        g.fSize(table_dir+"tabWeibullDiffShape_.tex",r"\scriptsize",1)
        dl(table_dir+"tabPowerDiffShape",[1],[],)
        me(table_dir+"tabPowerDiffShape_","\\bf",[(5,i) for i in    range(0,14)])
        g.fSize(table_dir+"tabPowerDiffShape_.tex",r"\scriptsize",1)
        dl(table_dir+"tabNormalDiffSamples",[1],[],)
        me(table_dir+"tabNormalDiffSamples_","\\bf",[(i,0) for i in         range(1,5)])
        g.fSize(table_dir+"tabNormalDiffSamples_.tex",r"\vspace{-.3cm}\scriptsize",1)
        dl(table_dir+"tabNormalDiffSamples2",[1],[],)
        me(table_dir+"tabNormalDiffSamples2_","\\bf",[(i,0) for i in         range(1,5)])
        g.fSize(table_dir+"tabNormalDiffSamples2_.tex","\\vspace{-.3cm}\\scriptsize",1)
        dl(table_dir+"tabUniformDiffSamples",[1],[],)
        me(table_dir+"tabUniformDiffSamples_","\\bf",[(i,0) for i in         range(1,5)])
        g.fSize(table_dir+"tabUniformDiffSamples_.tex",r"\vspace{-.3cm}\scriptsize",1)
        dl(table_dir+"tabUniformDiffSamples2",[1],[],)
        me(table_dir+"tabUniformDiffSamples2_","\\bf",[(i,0) for i in         range(1,5)])
        g.fSize(table_dir+"tabUniformDiffSamples2_.tex",r"\vspace{-.3cm}\scriptsize",1)
        dl(table_dir+"tabWeibullDiffSamples",[1],[],)
        me(table_dir+"tabWeibullDiffSamples_","\\bf",[(i,0) for i in         range(1,5)])
        g.fSize(table_dir+"tabWeibullDiffSamples_.tex",r"\vspace{-.3cm}\scriptsize",1)
        dl(table_dir+"tabPowerDiffSamples",[1],[],)
        me(table_dir+"tabPowerDiffSamples_","\\bf",[(i,0) for i in         range(1,5)])
        g.fSize(table_dir+"tabPowerDiffSamples_.tex",r"\vspace{-.3cm}\scriptsize",1)


    def makePreambule(self,NC,NE,NE2,NB,aux_dir):
        fname=aux_dir+"preambule1.tex"
        preambule=r"""The number of comparisons is $N_c={}$,
        each with the sample sizes of $n={}$ and $n'={}$.
        Each histogram have $N_b={}$ equally spaced bins.""".format(NC,NE,NE2,NB)
        with open(fname,"w") as f:
            f.write(preambule)
    def makeNormalVerification(self,NC,NE,NE2,NB,table_dir):
        check("antes")
        dists=[kolmogorovSmirnovDistance(
                n.random.normal(0,1,NE),n.random.normal(0,1,NE2))
                for i in range(NC)]; check("normal1")
        dists2=[kolmogorovSmirnovDistance(
                n.random.normal(3,2,NE),n.random.normal(3,2,NE2))
                for i in range(NC)]; check("normal2")
        dists3=[kolmogorovSmirnovDistance(
                n.random.normal(6,3,NE),n.random.normal(6,3,NE2))
                for i in range(NC)]; check("normal3")
        labelsh=(r"$\alpha N_c$",r"$\alpha$",r"$c(\alpha)$",r"$|C_1(\alpha)|$",r"$|C_2(\alpha)|$",r"$|C_3(\alpha)|$")
        caption=r"""The theoretical maximum number $\alpha N_c$ of rejections
        of the null hypothesis for significance levels $\alpha$.
        The $c_1$ values were calculated using simulations of normal distributions with $\mu={}$ and $\sigma={}$.
        The $c_2$ values were calculated using simulations of normal distributions with $\mu={}$ and $\sigma={}$.
        The $c_3$ values were calculated using simulations of normal distributions with $\mu={}$ and $\sigma={}$.
        Over all $N_c$ comparisons,
         $\mu(c_1)={:.4f}$ and $\sigma(c_1)={:.4f}$,
         $\mu(c_2)={:.4f}$ and $\sigma(c_2)={:.4f}$,
         $\mu(c_3)={:.4f}$ and $\sigma(c_3)={:.4f}$ .
        """.format(        0,1,
                3,2,
                6,3,
                   n.mean(dists ),n.std(dists ),
                   n.mean(dists2),n.std(dists2),
                   n.mean(dists3),n.std(dists3),
                   )
        data=[]
        labels=[]
        for alpha, calpha in zip(self.alphas,self.calphas):
            n1=sum([dist>calpha for dist in dists])
            n2=sum([dist>calpha for dist in dists2])
            n3=sum([dist>calpha for dist in dists3])
            data.append((alpha,calpha,n1,n2,n3))
            labels.append(alpha*NC)
        fname="tabNormNull.tex"
        lTable(labels,labelsh,data,caption,table_dir+fname)
        check("table {} written at {}".format(fname,table_dir))
    def makeUniformVerification(self,NC,NE,NE2,NB,table_dir):
        check("antes")
        dists=[kolmogorovSmirnovDistance(
                n.random.random(NE),n.random.random(NE2))
                for i in range(NC)]; check("uniforme1")
        dists2=[kolmogorovSmirnovDistance(
                2*n.random.random(NE)+2,2*n.random.random(NE2)+2)
                for i in range(NC)]; check("uniforme2")

        dists3=[kolmogorovSmirnovDistance(
                3*n.random.random(NE)+4,3*n.random.random(NE2)+4)
                for i in range(NC)]; check("uniforme3")
        labelsh=(r"$\alpha N_c$",r"$\alpha$",r"$c(\alpha)$",r"$|C_1(\alpha)|$",r"$|C_2(\alpha)|$",r"$|C_3(\alpha)|$")
        caption=r"""The theoretical maximum number $\alpha N_c$ of rejections
        of the null hypothesis for critical values of $\alpha$.
        The $c_1$ values were calculated using simulations of uniform distributions within $[{},{})$.
        The $c_2$ values were calculated using simulations of uniform distributions within $[{},{})$.
        The $c_3$ values were calculated using simulations of uniform distributions with $\mu={}$ and $\sigma={}$.
        Over all $N_c$ comparisons,
         $\mu(c_1)={:.4f}$ and $\sigma(c_1)={:.4f}$,
         $\mu(c_2)={:.4f}$ and $\sigma(c_2)={:.4f}$,
         $\mu(c_3)={:.4f}$ and $\sigma(c_3)={:.4f}$ .
        """.format(
                0,1,
                2,6,
                4,10,
                   n.mean(dists ),n.std(dists ),
                   n.mean(dists2),n.std(dists2),
                   n.mean(dists3),n.std(dists3),
                   )
        data=[]
        labels=[]
        for alpha, calpha in zip(self.alphas,self.calphas):
            n1=sum([dist>calpha for dist in dists])
            n2=sum([dist>calpha for dist in dists2])
            n3=sum([dist>calpha for dist in dists3])
            data.append((alpha,calpha,n1,n2,n3))
            labels.append(alpha*NC)
        fname="tabUniformNull.tex"
        lTable(labels,labelsh,data,caption,table_dir+fname)
        print("table {} written at {}".format(fname,table_dir))
    def makeWeibullVerification(self,NC,NE,NE2,NB,table_dir):
        check("antes")
        dists=[kolmogorovSmirnovDistance(
                n.random.weibull(0.1,NE),n.random.weibull(0.1,NE2))
                for i in range(NC)]; check("weibull1")
        dists2=[kolmogorovSmirnovDistance(
                n.random.weibull(2,NE),n.random.weibull(2,NE2))
                for i in range(NC)]; check("weibull2")
        dists3=[kolmogorovSmirnovDistance(
                n.random.weibull(4,NE),n.random.weibull(4,NE2))
                for i in range(NC)]; check("weibull3")
        dists4=[kolmogorovSmirnovDistance(
                n.random.weibull(6,NE),n.random.weibull(6,NE2))
                for i in range(NC)]; check("weibull4")
        labelsh=(r"$\alpha N_c$",r"$\alpha$",r"$c(\alpha)$",r"$|C_1(\alpha)|$",r"$|C_2(\alpha)|$",r"$|C_3(\alpha)|$",r"$|C_4(\alpha)|$")
        caption=r"""The theoretical maximum number $\alpha N_c$ of rejections
        of the null hypothesis for critical values of $\alpha$.
        The $c_1$ values were calculated using simulations of 1-parameter Weibull distributions with $a={}$.
        The $c_2$ values were calculated using simulations of 1-parameter Weibull distributions with $a={}$.
        The $c_3$ values were calculated using simulations of 1-parameter Weibull distributions with $a={}$.
        Over all $N_c$ comparisons,
        The $N_o$ values of $c_4$ were calculated using simulations of
         1-parameter Weibull distributions with $a={}$.
        Over all $N_c$ comparisons,
         $\mu(c_1)={:.4f}$ and $\sigma(c_1)={:.4f}$,
         $\mu(c_2)={:.4f}$ and $\sigma(c_2)={:.4f}$,
         $\mu(c_3)={:.4f}$ and $\sigma(c_3)={:.4f}$ .
         $\mu(c_4)={:.4f}$ and $\sigma(c_4)={:.4f}$ .
        """.format(
                0.1,
                2,
                4,
                6,
                   n.mean(dists ),n.std(dists ),
                   n.mean(dists2),n.std(dists2),
                   n.mean(dists3),n.std(dists3),
                   n.mean(dists4),n.std(dists4),
                   )
        data=[]
        labels=[]
        for alpha, calpha in zip(self.alphas,self.calphas):
            n1=sum([dist>calpha for dist in dists])
            n2=sum([dist>calpha for dist in dists2])
            n3=sum([dist>calpha for dist in dists3])
            n4=sum([dist>calpha for dist in dists4])
            data.append((alpha,calpha,n1,n2,n3,n4))
            labels.append(alpha*NC)
        fname="tabWeibullNull.tex"
        lTable(labels,labelsh,data,caption,table_dir+fname)
        print("table {} written at {}".format(fname,table_dir))
    def makePowerVerification(self,NC,NE,NE2,NB,table_dir):
        check("antes")
        dists=[kolmogorovSmirnovDistance(
                n.random.power(0.3,NE),n.random.power(0.3,NE2))
                for i in range(NC)]; check("power1")
        dists2=[kolmogorovSmirnovDistance(
                n.random.power(1,NE),n.random.power(1,NE2))
                for i in range(NC)]; check("power2")
        dists3=[kolmogorovSmirnovDistance(
                n.random.power(2,NE),n.random.power(2,NE2))
                for i in range(NC)]; check("power3")
        dists4=[kolmogorovSmirnovDistance(
                n.random.power(3,NE),n.random.power(3,NE2))
                for i in range(NC)]; check("power4")
        dists4_=[kolmogorovSmirnovDistance(
                n.random.power(4,NE),n.random.power(4,NE2))
                for i in range(NC)]; check("power5")
        labelsh=(r"$\alpha N_c$",r"$\alpha$",r"$c(\alpha)$",r"$|C_1(\alpha)|$",r"$|C_2(\alpha)|$",r"$|C_3(\alpha)|$",r"$|C_4(\alpha)|$",r"$|C_5(\alpha)|$")
        caption=r"""The theoretical maximum number $\alpha N_c$ of rejections
        of the null hypothesis for critical values of $\alpha$.
        The $c_1$ values were calculated using simulations of power functions distributions with $a={}$.
        The $c_2$ values were calculated using simulations of power functions distributions with $a={}$.
        The $c_3$ values were calculated using simulations of power functions distributions with $a={}$.
        The $c_4$ values were calculated using simulations of power functions distributions with $a={}$.
        The $c_5$ values were calculated using simulations of power functions distributions with $a={}$.
        Over all $N_c$ comparisons,
         $\mu(c_1)={:.4f}$ and $\sigma(c_1)={:.4f}$,
         $\mu(c_2)={:.4f}$ and $\sigma(c_2)={:.4f}$,
         $\mu(c_3)={:.4f}$ and $\sigma(c_3)={:.4f}$ .
         $\mu(c_4)={:.4f}$ and $\sigma(c_4)={:.4f}$ .
         $\mu(c_5)={:.4f}$ and $\sigma(c_5)={:.4f}$ .
        """.format(
                0.3,
                1,
                2,
                3,
                4,
                   n.mean(dists ),n.std(dists ),
                   n.mean(dists2),n.std(dists2),
                   n.mean(dists3),n.std(dists3),
                   n.mean(dists4),n.std(dists4),
                   n.mean(dists4_),n.std(dists4_),
                   )
        data=[]
        labels=[]
        for alpha, calpha in zip(self.alphas,self.calphas):
            n1=sum([dist>calpha for dist in dists])
            n2=sum([dist>calpha for dist in dists2])
            n3=sum([dist>calpha for dist in dists3])
            n4=sum([dist>calpha for dist in dists4])
            n4_=sum([dist>calpha for dist in dists4_])
            data.append((alpha,calpha,n1,n2,n3,n4,n4_))
            labels.append(alpha*NC)
        fname="tabPowerNull.tex"
        lTable(labels,labelsh,data,caption,table_dir+fname)
        print("table {} written at {}".format(fname,table_dir))

    def makeNormalDifferencesDispersion(self,NC,NE,NE2,NB,table_dir):
        xxN=n.linspace(.5,2,16,endpoint=True)
        distsAllN=[[kolmogorovSmirnovDistance_(
                n.random.normal(0,xxx,NE),n.random.normal(0,1,NE2)) for i in range(NC)]
                for xxx in xxN]
        distsAllNC=[[i[0] for i in j] for j in distsAllN]
        dnns=[[i[2] for i in j] for j in distsAllN]
        distsAllN_=[(n.mean(dd),n.std(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllNC]
        i=0
        labels=xxN
        labelsh=[r"$\sigma$",r"$\mu(c)$",r"$\sigma(c)$",r"$min(c)$",r"$max(c)$","$D$",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        data=distsAllN_
        caption=r"""Measurements of $c$ through simulations
        with normal distributions.
        One normal distribution is fixed, with $\mu=0$ and $\sigma=1$,
        and compared agaist normal distributions with $\mu=0$
        and different values of $\sigma$."""
        data_=[]
        i=0
        for dists in distsAllNC:
            line=[]
            dnn=dnns[i]
            kline=[dnnNorm(0,1,0,xxN[i],-4,4), n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabNormDiffDispersion.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmDiff3_")
        print("table {} written at {}".format(fname,table_dir))

    def makeNormalDifferencesMean(self,NC,NE,NE2,NB,table_dir):
        xxN2=n.linspace(0,1,11,endpoint=True)
        labels=xxN2
        distsAllN2=[[kolmogorovSmirnovDistance_(
                n.random.normal(xxx,1,NE),n.random.normal(0,1,NE2)) for i in range(NC)]
                for xxx in xxN2]
        distsAllNC2=[[i[0] for i in j] for j in distsAllN2]
        dnns=[[i[2] for i in j] for j in distsAllN2]
        data2=[(n.mean(dd),n.std(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllNC2]
        data2_=[]
        i=0
        for dists in distsAllNC2:
            line=[]
            dnn=dnns[i]
            kline=[dnnNorm(0,1,xxN2[i],1,-4,8), n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data2_.append(list(data2[i])+kline+line); i+=1
        caption=r"""Measurements of $c$ through simulations
        with normal distributions.
        One normal distribution is fixed, with $\mu=0$ and $\sigma=1$,
        and compared agaist normal distributions with different values of $\mu$ and fixed $\sigma=1$."""

        labelsh=[r"$\mu$",r"$\mu(c)$",r"$\sigma(c)$","$min(c)$","$max(c)$","$D$",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabNormDiffMean.tex"
        lTable(labels,labelsh,data2_,caption,table_dir+fname,"kolmDiff3_")
        print("table {} written at {}".format(fname,table_dir))
    def makeUniformDifferencesDispersion(self,NC,NE,NE2,NB,table_dir):
        xx=n.arange(.70,1.35,0.05)
        labels=xx
        distsAll=[[kolmogorovSmirnovDistance_(
                xxx*n.random.random(NE),n.random.random(NE2)) for i in range(NC)]
                for xxx in xx]
        distsAllC=[[i[0] for i in j] for j in distsAll]
        dnns=[[i[2] for i in j] for j in distsAll]
        data=[(n.mean(dd),n.std(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd)) ) for dd in distsAllC]
        data_=[]
        i=0
        for dists in distsAllC:
            line=[]
            dnn=dnns[i]
            kline=[dnnUni(0,xx[i],0,1,0,1.5), n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        caption=r"""Measurements of $c$ through simulations
        with uniform distributions.
        One uniform distribution has the fixed domain $[0,1)$.
        The other uniform distribution in each comparison
        is also centered around 0.5,
        but spread over $b=b_u-b_l$ there $b_l$ and $b_u$ are the lower and upper boudaries."""

        labelsh=[r"$b$",r"$\mu(c)$",r"$\sigma(c)$","min(c)","max(c)","$D$",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabUniformDiffSpread.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmDiff3_")
        i=0
        check("table {} written at {}".format(fname,table_dir))

    def makeUniformDifferencesMean(self,NC,NE,NE2,NB,table_dir):
        xx=n.arange(.0,.65,0.05)
        labels=xx+.5
        distsAll=[[kolmogorovSmirnovDistance_(
                n.random.random(NE)+xxx,n.random.random(NE2)) for i in range(NC)]
                for xxx in xx]
        distsAllC=[[i[0] for i in j] for j in distsAll]
        dnns=[[i[2] for i in j] for j in distsAll]
        data=[(n.mean(dd),n.std(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllC]
        data_=[]
        i=0
        for dists in distsAllC:
            line=[]
            dnn=dnns[i]
            kline=[dnnUni(xx[i],1+xx[i],0,1,0,1.5), n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        caption=r"""Measurements of $c$ through simulations
        with uniform distributions.
        One uniform distribution has the fixed domain $[0,1)$.
        The other uniform distribution in each comparison
        have varied mean values but always
        spread over a fixed $b=b_u-b_l$ there $b_l$ and $b_u$ are the lower and upper boudaries."""

        labelsh=[r"$\mu$",r"$\mu(c)$",r"$\sigma(c)$","min(c)","max(c)","$D$",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabUniformDiffMean.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmDiff3_")
        i=0
        check("table {} written at {}".format(fname,table_dir))
    def makeWeibullDifferencesShape(self,NC,NE,NE2,NB,table_dir):
        #xx=n.hstack(([0.01],n.arange(.10,3.,0.2)))
        #xx=n.arange(.3,3.,0.2)
        xx=n.arange(.7,3.,0.2)
        labels=xx
        distsAllW=[[kolmogorovSmirnovDistance_(
                n.random.weibull(xxx,NE),n.random.weibull(1.5,NE2),30000) for i in range(NC)]

                for xxx in xx]
        distsAllWC=[[i[0] for i in j] for j in distsAllW]
        dnns=[[i[2] for i in j] for j in distsAllW]
        data=[(n.mean(dd),n.std(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllWC]
        data_=[]
        i=0
        for dists in distsAllWC:
            line=[]
            dnn=dnns[i]
            kline=[dnnWeib(1.5,xx[i],0.000001,10), n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        caption=r"""Measurements of $c$ through simulations
        with 1-parameter Weibull distributions.
        One Weibull distribution has the fixed shape parameter $a=1.5$.
        The other Weibull distribution in each comparison
        has varied values of $a$."""

        labelsh=[r"$a$",r"$\mu(c)$",r"$\sigma(c)$","min(c)","max(c)","$D$",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabWeibullDiffShape.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmDiff3_")
        check("table {} written at {}".format(fname,table_dir))
    def makePowerDifferencesShape(self,NC,NE,NE2,NB,table_dir):
        xx=n.arange(.7,2.7,0.2)
        labels=xx
        distsAllW=[[kolmogorovSmirnovDistance_(
                n.random.power(xxx,NE),n.random.power(1.5,NE2)) for i in range(NC)]

                for xxx in xx]
        distsAllWC=[[i[0] for i in j] for j in distsAllW]
        dnns=[[i[2] for i in j] for j in distsAllW]
        data=[(n.mean(dd),n.std(dd),
            ("{:.3f},"*3)[:-1].format(*min3(dd)),
            ("{:.3f},"*3)[:-1].format(*max3(dd))) for dd in distsAllWC]
        data_=[]
        i=0
        for dists in distsAllWC:
            line=[]
            dnn=dnns[i]
            kline=[dnnPower(1.5,xx[i],-10,10), n.mean(dnn), n.std(dnn)]
            for calpha in self.calphas:
                line.append(sum([dist>calpha for dist in dists])/NC)
            data_.append(list(data[i])+kline+line); i+=1
        caption=r"""Measurements of $c$ through simulations
        with power function distributions.
        One power distribution has the fixed exponent parameter $1-a=2.5$.
        The other power function distribution in each comparison
        has varied values of $a$."""
        labelsh=[r"$a$",r"$\mu(c)$",r"$\sigma(c)$","min(c)","max(c)","$D$",r"$\mu(D_{F,F'})$",r"$\sigma(D_{F,F'})$"]
        labelsh+=[r"$\overline{{C({})}}$".format(alpha) for alpha in self.alphas]
        fname="tabPowerDiffShape.tex"
        lTable(labels,labelsh,data_,caption,table_dir+fname,"kolmDiff3_")
        i=0
        check("table {} written at {}".format(fname,table_dir))
    def notes(self):
        print("""#############
## região crítica?
# hipótese nula?
# distribuições chi-square? lei de potência?
# Kolmogorov-Smirnov distances? https://pypi.python.org/pypi/powerlaw
# discrete cases? http://www.itl.nist.gov/div898/handbook/eda/section3/eda35g.htm
# KS statistics == KS distance?
# bootstrap resampling ... e ... ad hoc
# D statistic, p-value
# the Kolmogorov-Smirnov goodness-of-fit is the D statistic and the KS distance?
# medir Fisher information?

# Another advantage is that it is an exact test (the chi-square goodness-of-fit test depends on an adequate sample size for the approximations to be valid) http://www.itl.nist.gov/div898/handbook/eda/section3/eda35g.htm
# Note that although the K-S test is typically developed in the context of continuous distributions for uncensored and ungrouped data, the test has in fact been extended to discrete distributions and to censored and grouped data.
# We do not discuss those cases here.

# The logarithm transformation may help to overcome cases where the Kolmogorov test data does not seem to fit the assumption that it came from the normal distribution. https://en.wikipedia.org/wiki/Kolmogorov%E2%80%93Smirnov_test
# If one chooses a critical value of the test statistic Dα such that P(Dn > Dα) = α, then a band of width ±Dα around Fn(x) will entirely contain F(x) with probability 1 − α.


# boas referências: http://stats.stackexchange.com/questions/411/motivation-for-kolmogorov-distance-between-distributions
# Most Monte Carlo studies show that the Anderson-Darling test is more powerful than the Kolmogorov-Smirnov test. It is available in scipy.stats with critical values, and in statsmodels with approximate p-values:

# often not very sensitive in establishing distances between two distributions, and a similar EDF-based test gives a better performance. https://asaip.psu.edu/Articles/beware-the-kolmogorov-smirnov-test
# The Anderson-Darling (AD) test was developed in the 1950s as a weighted CvM test to overcome both of these problems. 
# from scipy.stats import anderson_ksamp
# Bootstrap resampling is conceptually and computationally simple, and the theory underlying the bootstrap guarantee that the resulting significance levels are unbiased for a wide range of situations.

# In R you can also do a bootstrapped KS test sekhon.berkeley.edu/matching/ks.boot.html which gets rid of the continuity requirement – Dr G http://stats.stackexchange.com/questions/13326/can-i-use-kolmogorov-smirnov-to-compare-two-empirical-distributions

# >>> sm.stats.normal_ad(x)
# (0.23016468240712129, 0.80657628536145665)

    # https://en.wikipedia.org/wiki/Anderson%E2%80%93Darling_test""")
    def readme(self):
        return "clone this repo https://github.com/ttm/kolmogorov-smirnov\n and use the path to render the tables in this class"



#import time, os, shutil, datetime, re, random, string
#
#import nltk as k
#import rdflib as r
#
#from urllib.parse import quote
#import percolation as P

import datetime, os, shutil
import percolation as P, gmane as G
import html.parser
parser=html.parser.HTMLParser()
U=parser.unescape
c=P.check

def makeRepo(list_data_struct,fpath,fname,comment,aname=None,created_at=None,scriptpath=None,umbrella_dir=None):
    if not created_at:
        created_at=datetime.datetime.now()
    if not aname:
        aname=fname.split("/")[-1].replace(".","-").replace("+","P")
    fname_=fname.split("/")[-1]
    PREFIX="https://raw.githubusercontent.com/OpenLinkedSocialData/{}master/".format(umbrella_dir)
    tg=P.rdf.makeBasicGraph([["po","gmane"],[P.rdf.ns.po,P.rdf.ns.gmane]],"Email messages linked data")
    nchars=0
    for mid in list_data_struct.message_ids:
        message=list_data_struct.messages[mid]
        imsg=P.rdf.IC([tg],P.rdf.ns.gmane.Message,"{}".format(G.utils.urifyID(mid)))
#        if imsg == "http://purl.org/socialparticipation/gmane/Message#200406231741.i5NHfZbO019592_-AT-_sirius.codesourcery.com":
#            print("AQUIII")
        uris=[P.rdf.ns.gmane.mid,P.rdf.ns.gmane.sentAt,P.rdf.ns.gmane.body,P.rdf.ns.gmane.uid]
        tbody=U(message[3])
        nchars+=len(tbody)
        data=[U(mid),message[2],tbody,message[0]]
#        P.utils.breakMe()
        P.rdf.link([tg],imsg,mid,uris,data,draw=False)

        #ind=P.rdf.IC([tg],P.rdf.ns.gmane.Participant,"{}---{}".format(aname,message[0].replace("@","--AT--").replace(">","").replace("<","")))
#        uris=[P.rdf.ns.gmane.author]
#        data=[message[0]]
#        P.rdf.link([tg],ind,message[0],uris,data,draw=False)

        ind=P.rdf.IC([tg],P.rdf.ns.gmane.Participant,"{}".format(G.utils.urifyID(message[0])))
        uris=[P.rdf.ns.gmane.author]
        uris2=[ind]
        if message[1]:
            uris +=[P.rdf.ns.gmane.responseTo]
            imsg2=P.rdf.IC([tg],P.rdf.ns.gmane.Message,"{}".format(G.utils.urifyID(message[1])))
            uris2+=[imsg2]
        P.rdf.link_([tg],imsg,mid,uris,uris2,draw=False)
    tg2=P.rdf.makeBasicGraph([["po","gmane"],[P.rdf.ns.po,P.rdf.ns.gmane]],"Metadata for the snapshot of email messages")
    ind=P.rdf.IC([tg2],P.rdf.ns.po.Snapshot,
            aname,"Snapshot {}".format(aname))
    # escrever tb:
#    P.utils.breakMe()
    nmsgs=list_data_struct.n_messages
    nresponses=sum([i[1]!=None for i in list_data_struct.messages.values()])
    nusers=list_data_struct.n_authors
    # nchars
#    repourl="https://github.com/OpenLinkedSocialData/{}".format(aname)
    repourl="https://github.com/OpenLinkedSocialData/{}tree/master/{}".format(umbrella_dir,aname)
    metaurl="{}rdf/{}Meta.owl".format(PREFIX,aname)
    P.rdf.link([tg2],ind,"Snapshot {}".format(aname),
                          [P.rdf.ns.po.createdAt,
                          P.rdf.ns.po.triplifiedIn,
                          P.rdf.ns.po.donatedBy,
                          P.rdf.ns.po.availableAt,
#                          P.rdf.ns.po.originalFile,
                          P.rdf.ns.po.rdfFile,
                          P.rdf.ns.po.ttlFile,
                          P.rdf.ns.po.discorveryRDFFile,
                          P.rdf.ns.po.discoveryTTLFile,
                          P.rdf.ns.po.acquiredThrough,
                          P.rdf.ns.rdfs.comment,
                          P.rdf.ns.gmane.gmaneID,
                          P.rdf.ns.gmane.nParticipants,
                          P.rdf.ns.gmane.nMessages,
                          P.rdf.ns.gmane.nResponses,
                          P.rdf.ns.gmane.nCharacters,
                          ],
                          [created_at,
                           datetime.datetime.now(),
                           "Gmane database and public list participants",
                           repourl,
#                           "https://raw.githubusercontent.com/ttm/{}/master/base/{}".format(aname,fname.split("/")[-1]),
                           "{}rdf/{}Translate.owl".format(PREFIX,aname),
                           "{}rdf/{}Translate.ttl".format(PREFIX,aname),
                                metaurl,
                                "{}rdf/{}Meta.ttl".format(PREFIX,aname),
                           "Gmane public database of email lists",
                                comment,
                               fname_,
                               nusers,
                               nmsgs,
                               nresponses,
                               nchars
                           ])
    tg_=[tg[0]+tg2[0],tg[1]]
    fpath_="{}{}/".format(fpath,aname)
    c("before write")
    P.rdf.writeAll(tg_,aname+"Translate",fpath_,False,1,sizelimit=100000)

    if not os.path.isdir(fpath_+"scripts"):
        os.mkdir(fpath_+"scripts")
    shutil.copy(scriptpath,fpath_+"scripts/")
    if not os.path.isdir(fpath_+"base"):
        os.mkdir(fpath_+"base")
#    shutil.copy("{}/*".format(fname),fpath_+"base/")
    c("before zip")
    #P.utils.zipDir2(fname,fpath_+"base/msgs")
    c("before second write")
    P.rdf.writeAll(tg2,aname+"Meta",fpath_,False)

    # faz um README
    dates=[i for i in tg_[0].query(r"SELECT ?p WHERE {?s gmane:sentAt ?p} ORDER BY ASC(?p)")]
    #return tg_
    date1=dates[0][0].value
    date2=dates[-1][0].value
    #return tg_
    #nicks=queryMe(tg_[0],"SELECT ?s ?o WHERE {?s irc:nick ?o}")

    c("before queries")
    #nnicks=P.utils.countMe( tg_[0], "gmane:author")
    #nicks= P.utils.getAll2(  tg_[0],"gmane:author")
    #nicks_=[i.split("@")[-1] for i in nicks]

    #nreplies= P.utils.countMe(tg_[0],   "gmane:responseTo")
    #nmsgs=    P.utils.countMe(  tg_[0], "gmane:body")
    # nchars
    c("before queries")
    with open(fpath_+"README","w") as f:
        f.write("""This repo delivers RDF data from
the email list with id {} in the Gmane public database.
Collected around {}, the {} messages, 
of which {} are replies with a total of {} chars,
concerning {} participants, span from {} to {}.
The linked data is available at rdf/ and was
generated by the routine in the script/ directory.
Metadata for discovery is in file:
{}
All files should be available at the git repository:
{}
\n""".format(
            fname_,created_at,
            nmsgs,nresponses,nchars,nusers,
            date1,date2,
            repourl,metaurl
            ))
    return tg_





# função que faz todas as etapas
# de construção da rede
# e entrega os objetos certinho
import gmane as g, time, numpy as n, re, nltk as k, collections as c, string, pickle, os, langid, shutil
from nltk.corpus import wordnet as wn
import builtins as B
TT=time.time()
def check(amsg="string message"):
    global TT
    print(amsg, time.time()-TT); TT=time.time()
B.me=[]
B.tt_=[]
B.tt=[]
B.degen=[]
B.nonenglish=[]
puncts=set(string.punctuation)
#w=open("./wordsEn.txt","r")
#w=w.read()
#WL=w.split()
WL2=k.corpus.words.words()
#w=open("./wordlist.txt","r")
this_dir, this_filename = os.path.split(__file__)
#DATA_PATH = os.path.join(this_dir, "data", "data.txt")
w=open(os.path.join(this_dir,"words.txt"),"r")
# https://raw.githubusercontent.com/dwyl/english-words/master/words.txt
w=w.read()
WL=w.split()
labelsh=("","g.","p.","i.","h.")

#WL.append("email")
#WL.append("e-mail")
#WL.append("having")
WL_=set(WL)
WLP=k.corpus.floresta.words()
WLP_=set(WLP)
stopwords=set(k.corpus.stopwords.words("english"))
stopwordsP=set(k.corpus.stopwords.words("portuguese"))
f=open(os.path.join(this_dir,"pickledir/brill_taggerT2M1"),"rb")
brill_tagger=pickle.load(f)
f.close()
DL=g.tableHelpers.dl
ME=g.tableHelpers.me
replacement_patterns = [
(r'won\'t', 'will not'),
(r'can\'t', 'can not'),
(r'i\'m', 'i am'),
(r'ain\'t', 'is not'),
(r'(\w+)\'ll', '\g<1> will'),
(r'(\w+)n\'t', '\g<1> not'),
(r'(\w+)\'ve', '\g<1> have'),
(r'(\w+)\'s', '\g<1> is'),
(r'(\w+)\'re', '\g<1> are'),
(r'(\w+)\'d', '\g<1> would')
]
class RegexpReplacer(object):
    def __init__(self, patterns=replacement_patterns):
        self.patterns = [(re.compile(regex), repl) for (regex, repl) in patterns]
    def replace(self, text):
        s = text
        count_=0
        for (pattern, repl) in self.patterns:
            (s, count) = re.subn(pattern, repl, s)
            count_+=count
        return s, count_
REPLACER=RegexpReplacer()
R=REPLACER.replace
def pDump(tobject,tfilename):
    with open(tfilename,"wb") as f:
        pickle.dump(tobject,f,-1)

class EmptyClass:
    pass
    
class EmailStructures:
    """Class that makes all basic structures for a given email list"""
    def __init__(self,list_id,n_messages,text="yes",offset=0,basedir="~/.gmane3/"):
        TT=time.time()
        lid=list_id; TOTAL_M=n_messages
        lm=g.LoadMessages(lid,TOTAL_M,offset=offset, basedir=basedir)
        ds=g.ListDataStructures(lm,text="yes")
        print(lid+"{0:.2f} for loading messages and making datastructures".format(time.time()-TT)); TT=time.time()

        print(lid+"{0:.2f} for data structures".format(time.time()-TT)); TT=time.time()
        ts=g.TimeStatistics(ds)
        print("{0:.2f} for statistics along time".format(time.time()-TT)); TT=time.time()
        iN=g.InteractionNetwork(ds)
        nm=g.NetworkMeasures(iN,exclude=["rich_club"])
        print("{0:.2f} for network measures".format(time.time()-TT)); TT=time.time()
        if nm.N < 5:
            # network is too small
            # so make it disposable by makeTables_
            np2_ = EmptyClass()
            np2_.sectorialized_agents__ = [[], [], []]
        else:
            np2_=g.NetworkPartitioning(nm,2,"g")
            del np2_.binomial
        print("{0:.2f} for network partition".format(time.time()-TT)); TT=time.time()
        self.structs=lm, ds, ts, iN, nm, np2_
def perc_(alist):
    if type(alist) in (type([1,2]), type((2,4))):
        return [100*i/sum(alist[1:]) for i in alist]
    else:
        return 100*alist/alist[1:].sum()
def perc(alist):
    if type(alist) in (type([1,2]), type((2,4))):
        return [100*i/sum(alist) for i in alist]
    else:
        return 100*alist/alist.sum()
def digRoot(msgid,ds):
    layers=[[msgid]]
    while len(layers[-1]):
        layer=layers[-1]
        layers+=[[]]
        for mid in layer:
            if mid in ds.responses.keys():
                layers[-1]+=[i[0] for i in ds.responses[mid]]
    return layers,len(layers)

def generalMeasures(ds,np,ts):
    """Return overall measures from list datastructures and network partitioning"""
    #date1=ds.messages[ds.message_ids[0]][2].isoformat().split("T")[0]
    #date2=ds.messages[ds.message_ids[-1]][2].isoformat().split("T")[0]
    dt=ts.datetimes
    primeira,ultima=dt[0],dt[-1]
    date1=primeira.isoformat()[:-6]
    date2=ultima.isoformat(  )[:-6]
    deltaAnos=(ultima-primeira)
    deltaAnos_=deltaAnos.days/365.2425

    N=ds.n_authors
    Ns=[len(i) for i in np.sectorialized_agents__]
    Ns_=perc(Ns)
    M_=ds.n_messages-ds.n_empty
    M=ds.n_messages
    #Mh=sum([len(ds.author_messages[author]) for author in np.sectorialized_agents__[2]])
    #Mi=sum([len(ds.author_messages[author]) for author in np.sectorialized_agents__[1]])
    #Mp=sum([len(ds.author_messages[author]) for author in np.sectorialized_agents__[0]])
    #Ms=[Mh,Mi,Mp][::-1]
    Ms=[sum([len(ds.author_messages[i]) for i in j])
        for j in np.sectorialized_agents__]
    #M2=[100*i/ds.n_messages for i in Ms]
    Ms_=perc(Ms)
    NM=N/M
    NM_=100*NM
    NM_missing=M_/N
    NM_missing_=100*NM_missing
    NMs=[i/j if j!=0 else n.inf for i,j in zip(Ns,Ms)]
    NMs_=perc(NMs)
    #idsh=[i[0] for j in np.sectorialized_agents__[2] for i in ds.author_messages[j] if ds.messages[i[0]][1]==None]
    #idsi=[i[0] for j in np.sectorialized_agents__[1] for i in ds.author_messages[j] if ds.messages[i[0]][1]==None]
    #idsp=[i[0] for j in np.sectorialized_agents__[0] for i in ds.author_messages[j] if ds.messages[i[0]][1]==None]
    #idsh_=len(idsh)
    #idsi_=len(idsi)
    #idsp_=len(idsp)
    #ids=[idsh_,idsi_,idsp_][::-1]
    #ids_=[100*ii/Gamma for ii in ids]
    #Gamma=len([i for i in ds.message_ids if ds.messages[i][1]==None])
    Gammas=[sum([len([i for i in ds.author_messages[aid] if i[1]==None])
           for aid in sa]) for sa in np.sectorialized_agents__]
    Gammas_=perc(Gammas)
    G_=[100*i/j for i,j in zip(Gammas,Ms)]
    # Gammas_==ids_
    #roots=[[[i for i in ds.author_messages[aid] if i[1]==None]
    #           for aid in sa] for sa in pr.sectorialized_agents__]
    #roots_=[i for j in roots for i in j]
    ## *) a partir de cada uma delas, procura outras que tenham
    ## ela como resposta e assim por diante,
    ## até não achar mais resposta, guarda o número de mensagens
    ## encontradas
    #roots__=[[[i[j][0] for j in range(len(i))] for i in rr if i] for rr in roots]
    #rr=[]
    roots_sectors=[]
    tlength_sectors=[]
    threads_sectors=[]
    for setor in np.sectorialized_agents__:
        roots_sector=[]
        tlength_sector=[]
        threads_sector=[]
        for agentid in setor:
            messages=ds.author_messages[agentid]
            for message in messages:
                if message[1]==None: # nova thread, guarda ID
                    roots_sector.append(message[0])
                    t_sector,lsector=digRoot(message[0],ds)
                    tlength_sector.append(lsector)
                    threads_sector.append(t_sector)
        roots_sectors.append(roots_sector)
        tlength_sectors.append(tlength_sector)
        threads_sectors.append(threads_sector)
    mt=[n.mean(i) for i in tlength_sectors]
    st=[n.std(i) for i in tlength_sectors]
    tls=[i for j in tlength_sectors for i in j]
    mt_ =n.mean(tls)
    st_ =n.std(tls)
    mvars=("date1","date2","deltaAnos_",
            "N","Ns","Ns_","M","M_","Ms","Ms_",
            "NM","NM_","NMs","NMs_","NM_missing","NM_missing_",
            "Gammas","Gammas_","G_",
            "roots_sectors","tlength_sectors","threads_sectors",
            "mt","st","tls","mt_","st_")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
    #return date1,date2,N,Ns,Ns_,Ms,M2,Gamma,ids,ids_,M_,MN,MN_
def makeGeneralTable(generalMeasures_instance, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="geralInline.tex",tag=None):
    gms=generalMeasures_instance
    labelsh=("","g.","p.","i.","h.")
    labels=(r"$N$",r"$N_{\%}$",r"$M$",r"$M_{\%}$",
            r"$\Gamma$",r"$\Gamma_{\%}$",r"$\frac{\Gamma}{M}\%$",
            r"$\mu(\gamma)$",r"$\sigma(\gamma)$")
    N,Ns,Ns_,M,Ms,Ms_,Gammas,Gammas_,G_,mt_,mt,st_,st,deltaAnos_,date1,date2=[gms[i]
            for i in ("N","Ns","Ns_","M","Ms","Ms_","Gammas","Gammas_","G_","mt_","mt","st_","st","deltaAnos_","date1","date2")]
    Gamma=sum(Gammas)
    data=[[N]+Ns,[100]+Ns_,[M]+Ms,[100]+Ms_,[Gamma]+Gammas,[100]+Gammas_,[100*Gamma/M]+G_,[mt_]+mt,[st_]+st]
    caption=r"""Distribution of participants, messages and threads among each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, 
    {{\bf h.}} for hubs) in a total timespan of {:.2f} years (from {} to {}). $N$ is the number of participants, $M$ is the number of messages, $\Gamma$ is the number of threads, and $\gamma$ is the number of messages in a thread.
    The \% denotes the usual `per cent' with respecto to the total quantity ($100\%$ for {{\bf g.}})
    while $\mu$ and $\sigma$ denote mean and standard deviation.""".format(deltaAnos_,date1,date2)
    fname_=mkName(table_dir,fname,tag)
    g.tableHelpers.lTable(labels,labelsh,data,caption,fname_,"textGeral")
    dl=g.tableHelpers.dl
    me=g.tableHelpers.me
    me(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)])
    dl(fname_[:-4]+"_",[1],[1],list(range(2,8,2))+[8,9])
def mkName(tdir,fname,tag):
    return tdir+fname.replace(".tex","{}.tex".format(tag))



def makeText_(ds,pr):
    texts=[]
    msg_ids=[]
    textG=""
    for sector in pr.sectorialized_agents__:
        texts+=[""]
        msg_ids.append([])
        for author in sector:
            for message in ds.author_messages[author]:
                mid=message[0]
                text=ds.messages[mid][3]
                texts[-1]+=text
                textG+=text
                msg_ids[-1].append(mid)
    texts=[textG]+texts
    foo=[REPLACER.replace(i) for i in texts]
    texts_=[i[0] for i in foo]
    ncontractions=[i[1] for i in foo]
    return texts_,ncontractions, msg_ids
def makeText(ds,mid=None):
    if not mid:
        t=[ds.messages[i][3] for i in ds.message_ids]
    else:
        t=[ds.messages[i][3] for i in mid]
    T_="\n".join(t) # todo o texto, com contracoes
    T,ncontractions=REPLACER.replace(T_) # todo o texto, sem contracoes
    return T, ncontractions

def medidasTokensQ_(T,lang="en"):
    atime=time.time()
    wtok=k.tokenize.wordpunct_tokenize(T)
    wtok_=[t.lower() for t in wtok]
    if lang=="en":
        kw=[len(i) for i in wtok_ if i in WL_]
        sw=[len(i) for i in wtok_ if i in stopwords]
    else:
        kw=[len(i) for i in wtok_ if i in WLP_]
        sw=[len(i) for i in wtok_ if i in stopwordsP]
    mvars=("kw","sw")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def medidasTokensQ(T,lang="en"):
    atime=time.time()
    wtok=k.tokenize.wordpunct_tokenize(T)
    wtok_=[t.lower() for t in wtok]
    if lang=="en":
        kw=[len(i) for i in wtok_ if i in WL_]
        sw=[len(i) for i in wtok_ if i in stopwords]
    else:
        kw=[len(i) for i in wtok_ if i in WLP_]
        sw=[len(i) for i in wtok_ if i in stopwordsP]
    mkw=n.mean(kw)
    dkw=n.std(kw)
    msw=n.mean(sw)
    dsw=n.std(sw)
    mvars=("mkw","dkw","msw","dsw")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def makeSentencesTable(medidasSentencas_dict, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="sentencesInline.tex",tag=None):
    sms=medidasSentencas_dict
    mvars=("nsents",
            "Mchars_sents","Schars_sents",
            "Mtoks_sents","Stoks_sents",
            "Mknownw_sents","Sknownw_sents",
            "Mstopw_sents","Sstopw_sents",
            "Mpuncts_sents","Spuncts_sents",)
    sms_=[[sms[j][i] for j in range(4)] for i in mvars]
    labelsh=("","g.","p.","i.","h.")
    labels=(r"$sents$",r"$sents_{\%}$",
            r"$\mu_S(chars)$", r"$\sigma_S(chars)$",
            r"$\mu_S(tokens)$",r"$\sigma_S(tokens)$",
            r"$\mu_S(knownw)$",r"$\sigma_S(knownw)$",
            r"$\mu_S(stopw)$", r"$\sigma_S(stopw)$",
            r"$\mu_S(puncts)$",r"$\sigma_S(puncts)$",
            )
    caption=r"""Sentences sizes in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, {{\bf h.}} for hubs)."""
    #data=list(map(list, zip(*tms_)))
    data=sms_
    nsents=data[0]
    nsents_=perc_(nsents)
    data=n.array(data[1:])
    data=n.vstack((nsents,nsents_,data))
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels,labelsh,data,caption,fname_,"textGeral")
    ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)])
    DL(fname_[:-4]+"_",[1],[1],[2,4,6,8,10,12])

def makeTokenSizesTable(medidasTokens__instance, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="tokenSizesInline.tex",tag=None):
    tms=medidasTokens__instance
    mvars=("Mtoken","Stoken","Mknownw","Sknownw",
            "Mknownw_diff","Sknownw_diff",
            "Mstopw","Sstopw")
    tms_=[[tms[j][i] for j in range(4)] for i in mvars]
    labelsh=("","g.","p.","i.","h.")
    labels=( r"$\mu(\overline{tokens})$",r"$\sigma(\overline{tokens})$",
            r"$\mu(\overline{knownw})$",r"$\sigma(\overline{knownw})$",
            r"$\mu(\overline{knownw \neq})$",r"$\sigma(\overline{knownw \neq})$",
            r"$\mu(\overline{stopw})$",r"$\sigma(\overline{stopw})$")
    caption=r"""Token sizes in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, {{\bf h.}} for hubs)."""
    #data=list(map(list, zip(*tms_)))
    data=tms_
    #ntoks=data[0]
    #ntoks_=perc_(ntoks)
    #data=n.array(data[1:])
    #data=n.vstack((ntoks,ntoks_,data*100))
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels,labelsh,data,caption,fname_,"textGeral_")
    ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)])
    DL(fname_[:-4]+"_",[1],[1],[2,4,6,8])


def makeTokensTable(medidasTokens__instance, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="tokensInline.tex",tag=None):
    tms=medidasTokens__instance
    mvars=("tokens",
            "tokens_diff",
            "knownw",
            "knownw_diff",
            "stopw",
            "punct",
            "contract")
    tms_=[[tms[j][i] for j in range(4)] for i in mvars]
    labelsh=("","g.","p.","i.","h.")
    labels=(r"$tokens$",
            r"$tokens_{\%}$",
            r"$tokens \neq$",
            r"$\frac{knownw}{tokens}$",
            r"$\frac{knownw \neq}{knownw}$",
            r"$\frac{stopw}{knownw}$",
            r"$\frac{punct}{tokens}$",
            r"$\frac{contrac}{tokens}$",
            )
    caption=r"""tokens in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, 
    {{\bf h.}} for hubs)."""
    #data=list(map(list, zip(*tms_)))
    data=tms_
    ntoks=data[0]
    ntoks_=perc_(ntoks)
    data=n.array(data[1:])
    data=n.vstack((ntoks,ntoks_,data*100))
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels,labelsh,data,caption,fname_,"textGeral")
    ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)])
    DL(fname_[:-4]+"_",[1],[1],[2,3,5,7,8])
def chunks(l, n):
    """Yield successive n-sized chunks from l."""
    ll=[]
    for i in range(0, len(l), n):
        ll.append(l[i:i+n])
    return ll
def medidasSinais2_(medidas_pos_list,medidas_mensagens):
    return [medidasSinais2(post,mmU)
            for post,mmU in zip(medidas_pos_list,medidas_mensagens)]
def medidasSinais2(post,medidas_mensagensU):
    sinal=[[i[1] for i in j] for j in post["tags"]]
    sinal_=chunks([i[1] for j in post["tags"] for i in j],100)
    sinais={}
    sinais["adj"]=[j.count("ADJ") for j in sinal]
    sinais["sub"]=[j.count("NOUN") for j in sinal]
    sinais["pun"]=[j.count(".") for j in sinal]
    sinais["verb"]=[j.count("VERB") for j in sinal_]
    sinais["chars"]=medidas_mensagensU["toks_msgs"]
    return sinais

def medidasSinais_(TS):
    return [medidasSinais(T) for T in TS]
def medidasSinais(T):
    wtok=k.tokenize.wordpunct_tokenize(T)
    lens_tok=[len(i) for i in wtok]
    lens_word=[len(i) for i in wtok if (i not in stopwords) and (i in WL_)]
    lens_sent=[len(i) for i in k.sent_tokenize(T)]
    mvars=("lens_tok","lens_word","lens_sent")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def ksAll(sigDict,mkeys):
    l=[]
    for key in mkeys:
        l.append([])
        for i in range(4):
            l[-1].append([])
            for j in range(4):
                vals=g.ksStatistics.kolmogorovSmirnovDistance__(sigDict[i][key],sigDict[j][key])
                l[-1][-1].append(vals)
    return l
def makeKSTables(dists,table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fnames=None,tags=None,tag=None):
    ldists=[]
    for dists_meas in dists:
        l=[]
        for sect1_meas in dists_meas:
            calphas=[]
            dnns=[]
            for sect2_val in sect1_meas:
               calpha,dnn=sect2_val 
               calphas+=[calpha]
               dnns+=[dnn]
            l+=[calphas,dnns]
        ldists.append(l) # new table
    dists=ldists 
    labels=labelsh[1:]
    labels_=[(l,"") for l in labels]
    labels__=[i for j in labels_ for i in j]
    caption="KS distances on {}."
    count=0
    if not fnames:
        fnames=[str(i) for i in range(len(dists))]
    if not tags:
        tags=[str(i) for i in range(len(dists))]
    for meas,fname,tag_ in zip(dists,fnames,tags):
        fname_=mkName(table_dir,fname+".tex",tag)
        g.lTable(labels__,labelsh,meas,caption.format(tag_),
                fname_,"ksDistances")
        ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)]+[(i,0) for i in range(1,9)])
        DL(fname_[:-4]+"_",[1],[1],[2,4,6,8])
#        ME(fname_+"_","\\bf",,1)


def medidasTokens__(lt=("texts",),ct=("ncontractions",)):
    return [medidasTokens_(i,j) for i,j in zip(lt,ct)]
def medidasTokens_(T,ncontract=None):
    wtok=k.tokenize.wordpunct_tokenize(T)
    wtok_=[t.lower() for t in wtok]
    tokens=len(wtok) #
    tokens_=set(wtok)
    tokens_diff=len(tokens_)/tokens # 
    punct=sum([sum([tt in puncts for tt in t])==len(t) for t in wtok_])
    punct/=tokens
    known=[i for i in wtok_ if (i not in stopwords) and (i in WL_)]
    knownw=len(known)
    known_=set(known)
    knownw_diff=len(known_)/knownw
    stop=[i for i in wtok_ if i in stopwords]
    stopw=len(stop)/knownw
    knownw/=tokens
    contract=ncontract/tokens

    # media e desvio de tamanhos:
    # tokens,
    Mtoken,Stoken=mediaDesvio_(wtok_)
    # known words sem stop,
    Mknownw,Sknownw=mediaDesvio_(known)
    # known words sem stop e sem repetição
    Mknownw_diff,Sknownw_diff=mediaDesvio_(known_)
    # stop words
    Mstopw,Sstopw=mediaDesvio_(stop)

    #stokwn=[len(i) for i in known]
    #skownw=[len(i) for i in known]
    mvars=("tokens","tokens_diff","punct",
            "knownw","knownw_diff","stopw","contract",
            "Mtoken","Stoken","Mknownw","Sknownw",
            "Mknownw_diff","Sknownw_diff",
            "Mstopw","Sstopw")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict

def medidasTokens(T):
    atime=time.time()
    wtok=k.tokenize.wordpunct_tokenize(T)
    wtok_=[t.lower() for t in wtok]
    nt=len(wtok) #
    ntd=len(set(wtok)) # 
    # tokens que sao pontuacoes
    ntp=sum([sum([tt in puncts for tt in t])==len(t) for t in wtok]) #
    # known and unkown words
    kw=[] #
    ukw=[] #
    tp=[]
    sw=[]
    for t in wtok_:
        if t in WL_:
            kw.append(t)
        elif sum([tt in puncts for tt in t])==len(t):
            tp.append(t)
        else:
            ukw.append(t)
        if t in stopwords:
            sw.append(t)
    sw_=set(sw)
    kw_=set(kw)
    ukw_=set(ukw)
    kwss=[i for i in kw if wn.synsets(i)] #
    kwss_=set(kwss) #
    # known words that does not have synsets
    kwnss=[i for i in kw if i not in kwss_] #
    print("MT2:", atime-time.time()); atime=time.time()
    kwnss_=set(kwnss) #
    # words that are stopwords
    kwsw=[i for i in kw if i in stopwords] #
    kwsw_=set(kwsw)
    print("MT3:", atime-time.time()); atime=time.time()
    # known words that are not stopwords
    kwnsw=[i for i in kw if i not in stopwords] #
    kwnsw_=set(kwnsw) #
    # unknown words that are stopwords
    ukwsw=[i for i in ukw if i in stopwords] #
    print( "MT4:", atime-time.time()); atime=time.time()
    # known words that return synsets and are stopwords
    kwsssw=[i for i in kwss if i in stopwords] #
    print("MT5:", atime-time.time()); atime=time.time()
    # known words that dont return synsets and are stopwords
    kwnsssw=[i for i in kwnss if i in stopwords] #
    print("MT6:", atime-time.time()); atime=time.time()
    # words that are known, are not stopwords and do not return synset
    foo_=kwnss_.difference(stopwords)
    kwnssnsw=[i for i in kw if i in foo_] #
    print("MT7:", atime-time.time()); atime=time.time()
    foo_=kwss_.difference(stopwords) 
    kwssnsw=[i for i in kw if i in foo_] #
    kwssnsw_=set(kwssnsw)
    print("MT8:", atime-time.time()); atime=time.time()
    mvars=("nt", "ntd", "ntp","sw","sw_","kw", "kw_", "tp", "ukw", "ukw_", "kwss", "kwss_", "kwnss", "kwnss_","kwsw", "kwsw_", "kwnsw", "kwnsw_", "ukwsw", "kwsssw", "kwnsssw", "kwnssnsw", "kwssnsw","kwssnsw_")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict

def makeCharTable(charsMeasures_instance, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="charsInline.tex",tag=None):
    cms=charsMeasures_instance
    labelsh=("","g.","p.","i.","h.")
    labels=(r"$chars$",
            r"$chars_{\%}$",
            r"$\frac{spaces}{chars}$",
            r"$\frac{punct}{chars-spaces}$",
            r"$\frac{digits}{chars-spaces}$",
            r"$\frac{letters}{chars-spaces}$",
            r"$\frac{vogals}{letters}$",
            r"$\frac{uppercase}{letters}$",
            )

    caption=r"""Characters in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, 
    {{\bf h.}} for hubs)."""
    data=list(map(list, zip(*cms)))
    nchars=data[0]
    nchars_=perc_(nchars)
    data=n.array(data[1:])
    data=n.vstack((nchars,nchars_,data*100))
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels,labelsh,data,caption,fname_,"textGeral")
    ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)])
    DL(fname_[:-4]+"_",[1],[1],[2,4,5,7,8])
def medidasLetras_(LT=["list","of","strings"]):
        return [medidasLetras(i) for i in LT]
def medidasLetras(T):
    # quantos caracteres
    nc=len(T)
    # espacos
    ne=T.count(" ")
    # letras (e maiusculas)
    nl=sum([t.isalpha() for t in T])
    nm=sum([t.isupper() for t in T])
    # vogais
    nv=sum([t in ("a","e","i","o","u") for t in T])
    # pontuacao
    np=sum([t in puncts for t in T])
    # numerais
    nd=sum([t.isdigit() for t in T])
    return nc,ne/nc,np/(nc-ne),nd/(nc-ne),nl/(nc-ne),nv/nl,nm/nl

def mediaDesvio_(medidas):
    medidas_=[len(i) for i in medidas]
    return n.mean(medidas_),n.std(medidas_)
def mediaDesvio(tid="astring",adict={"stringkey":"tokens"}):
    tid_=tid+"_"
    toks=[len(i) for i in adict[tid]]
    toks_=[len(i) for i in adict[tid_]]

    mtid=n.mean(toks)
    dtid=n.std(toks)
    mtid_=n.mean(toks_)
    dtid_= n.std(toks_)

    fdict={}
    fdict["m"+tid]=mtid
    fdict["d"+tid]=dtid
    fdict["m"+tid_]=mtid_
    fdict["d"+tid_]=dtid_

    return fdict
def medidasTamanhosTokens(medidas_tokens):
    mdict={}
    MT=medidas_tokens
    mdict.update(mediaDesvio("kw",MT))
    mdict.update(mediaDesvio("kwnsw",MT))
    mdict.update(mediaDesvio("kwssnsw",MT))
    mdict.update(mediaDesvio("kwssnsw",MT))
    mdict.update(mediaDesvio("kwsw",MT))
    mdict.update(mediaDesvio("sw",MT))
    return mdict
def makeCorrelationTable_(measures_pca, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="correlationInline.tex",tag=None):
    mp=measures_pca
    cors=[i["pca"].C for i in mp]
    cors_=[]
    for secn in range(len(cors[0])):
        for cor in cors: # cor correlation measure
            cors_.append(cor[secn])
    data=cors_
    labels=mp[0]["vlabels"]
    labelsh=[""]+labels
    labels_=[(i,"","","") for i in labels]
    labels__=[i for j in labels_ for i in j]
    labels__[1:4]=["(p.)","(i.)","(h.)"]
    caption="Pierson correlation coefficient for the topological and textual measures."
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels__,labelsh,data,caption,fname_,"textCorr")
    # renderiza matriz como tabela
    #ME(table_dir+fname[:-4],"\\bf",[(0,i) for i in range(1,5)])
    nz=(n.abs(n.array(data))>.6).nonzero()
    ii=nz[0]
    jj=nz[1]
    #pts=[(i,j) for i,j in zip(ii,jj)]
    pts=[(i+1,j+1) for i,j in zip(ii,jj)]
    B.thing=nz,data,pts
    ME(fname_[:-4],"\\bf",pts)
    DL(fname_[:-4]+"_",[1],[1],[2,3,4,
                                         6,7,8,
                                         10,11,12,
                                         14,15,16,
                                         18,19,20,
                                         22,23,24,
                                         26,27,28,
                                         30,31,32,
                                         34,35,36])


def makeCorrelationTable(correlationMatrix, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="correlationInline.tex",mvars=[]):
    #mvars=[i.replace("_","") for i in mvars]
    labelsh=[""]+mvars
    labels=mvars
    caption=r"""Correlation of textual and topological metrics."""
    data=correlationMatrix*100
    g.lTable(labels,labelsh,data,caption,table_dir+fname,"textCorr")
#    ME(table_dir+fname[:-4],"\\bf",[(0,i) for i in range(1,5)])
#    DL(table_dir+fname[:-4]+"_",[1],[1],[])
    #DL(table_dir+fname[:-4],[1],[1],[],0)



def makeMessagesTable(medidasMensagens_dict, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="messagesInline.tex",tag=None):
    mms=medidasMensagens_dict
    mvars=("nmsgs",
            "Msents_msgs","Ssents_msgs",
            "Mtokens_msgs","Stokens_msgs",
            "Mknownw_msgs","Sknownw_msgs",
            "Mstopw_msgs","Sstopw_msgs",
            "Mpuncts_msgs","Spuncts_msgs",
            "Mchars_msgs","Schars_msgs",
            )
    mms_=[[mms[j][i] for j in range(4)] for i in mvars]
    labelsh=("","g.","p.","i.","h.")
    labels=(r"$msgs$",r"$msgs_{\%}$",
            r"$\mu_M(sents)$", r"$\sigma_M(sents)$",
            r"$\mu_M(tokens)$",r"$\sigma_M(tokens)$",
            r"$\mu_M(knownw)$",r"$\sigma_M(knownw)$",
            r"$\mu_M(stopw)$", r"$\sigma_M(stopw)$",
            r"$\mu_M(puncts)$",r"$\sigma_M(puncts)$",
            r"$\mu_M(chars)$", r"$\sigma_M(chars)$",
            )

    caption=r"""Messages sizes in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, {{\bf h.}} for hubs)."""
    #data=list(map(list, zip(*tms_)))
    data=mms_
    nmsgs=data[0]
    nmsgs_=perc_(nmsgs)
    data=n.array(data[1:])
    data=n.vstack((nmsgs,nmsgs_,data))
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels,labelsh,data,caption,fname_,"textGeral")
    ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)])
    DL(fname_[:-4]+"_",[1],[1],[2,4,6,8,10,12,14,16])


def medidasMensagens_(ds,msg_ids):
    return [medidasMensagens(ds,mids) for mids in [None]+list(msg_ids)]
def medidasMensagens(ds,tids=None):
    # TTM
    if not tids:
        mT=[ds.messages[i][3] for i in ds.message_ids]
    else:
        mT=[ds.messages[i][3] for i in tids]
    tokens_msgs=[k.tokenize.wordpunct_tokenize(t) for t in mT] # tokens
    knownw_msgs=[[i for i in toks if (i not in stopwords) and (i in WL_)] for toks in tokens_msgs]
    stopw_msgs=[[i for i in toks if i in stopwords] for toks in tokens_msgs]
    puncts_msgs=[[i for i in toks if
         (len(i)==sum([(ii in puncts) for ii in i]))]
         for toks in tokens_msgs] #
    sents_msgs=[k.sent_tokenize(t) for t in mT] # tokens
    nmsgs=len(mT)
    toks_msgs=[len(i) for i in mT]
    Mchars_msgs,   Schars_msgs  = mediaDesvio_(mT)
    Mtokens_msgs,  Stokens_msgs = mediaDesvio_(tokens_msgs)
    Mknownw_msgs,  Sknownw_msgs = mediaDesvio_(knownw_msgs)
    Mstopw_msgs,   Sstopw_msgs  = mediaDesvio_(stopw_msgs)
    Mpuncts_msgs,  Spuncts_msgs = mediaDesvio_(puncts_msgs)
    Msents_msgs,Ssents_msgs     = mediaDesvio_(sents_msgs)
    mvars=("nmsgs","toks_msgs",
            "Msents_msgs","Ssents_msgs",
            "Mtokens_msgs","Stokens_msgs",
            "Mknownw_msgs","Sknownw_msgs",
            "Mstopw_msgs","Sstopw_msgs",
            "Mpuncts_msgs","Spuncts_msgs",
            "Mchars_msgs","Schars_msgs",
            )
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def medidasSentencas_(Ts=['list',"of","strings"]):
    return [medidasSentencas(i) for i in Ts]
def medidasSentencas(T):
    TS=k.sent_tokenize(T)
    tokens_sentences=[k.tokenize.wordpunct_tokenize(i) for i in TS] ### Para os POS tags
    knownw_sentences=[[i for i in ts if (i not in stopwords) and (i in WL_)] for ts in tokens_sentences]
    stopw_sentences =[[i for i in ts if i in stopwords] for ts in tokens_sentences]
    puncts_sentences=[[i for i in ts if
         (len(i)==sum([(ii in puncts) for ii in i]))]
         for ts in tokens_sentences] #
    Mchars_sents,  Schars_sents  = mediaDesvio_(TS)
    Mtoks_sents,   Stoks_sents   = mediaDesvio_(tokens_sentences)
    Mknownw_sents, Sknownw_sents = mediaDesvio_(knownw_sentences)
    Mstopw_sents,  Sstopw_sents  = mediaDesvio_(stopw_sentences)
    Mpuncts_sents, Spuncts_sents = mediaDesvio_(puncts_sentences)
    nsents=len(TS)
    mvars=("Mchars_sents","Schars_sents",
            "Mtoks_sents","Stoks_sents",
            "Mknownw_sents","Sknownw_sents",
            "Mstopw_sents","Sstopw_sents",
            "Mpuncts_sents","Spuncts_sents","nsents",
            "tokens_sentences")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict



def medidasTamanhosSentencas(T,medidas_tokens):
    MT=medidas_tokens
    ############
    # medidas de sentencas
    TS=k.sent_tokenize(T)
    # media e desvio de numero de caracteres por sentenca
    tTS=[len(i) for i in TS]
    mtTS=n.mean(tTS) #
    dtTS=n.std(tTS) #
    
    # media e desvio do tamanho das sentencas em tokens
    sTS=[k.tokenize.wordpunct_tokenize(i) for i in TS] ### Para os POS tags
    tsTS=[len(i) for i in sTS]
    mtsTS=n.mean(tsTS) #
    dtsTS=n.std(tsTS) #

    # media e desvio do tamanho das sentencas em palavras conhecidas
    kw_=MT["kw_"]
    tsTSkw=[len([ii for ii in i if ii in kw_]) for i in sTS]
    mtsTSkw=n.mean(tsTSkw) #
    dtsTSkw=n.std(tsTSkw) #

    # media e desvio do tamanho das sentencas em palavras que retornam synsets e nao sao stopwords
    pv_=MT["kwssnsw_"]
    tsTSpv=[len([ii for ii in i if ii in pv_]) for i in sTS]
    mtsTSpv=n.mean(tsTSpv) #
    dtsTSpv=n.std(tsTSpv) #

    mvars=("mtTS","dtTS","mtsTS","dtsTS","mtsTSkw","dtsTSkw",
           "mtsTSpv","dtsTSpv","sTS")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def medidasTamanhosMensagens(ds, tids=None):
    if not tids:
        mT=[ds.messages[i][3] for i in ds.message_ids]
    else:
        mT=[ds.messages[i][3] for i in tids]

    tmT=[len(t) for t in mT] # chars
    ttmT=[len(k.tokenize.wordpunct_tokenize(t)) for t in mT] # tokens
    tsmT=[len(k.sent_tokenize(t)) for t in mT] # sentences

    mtmT=n.mean(tmT)
    dtmT=n.std(tmT)
    mttmT=n.mean(ttmT)
    dttmT=n.std(ttmT)
    mtsmT=n.mean(tsmT)
    dtsmT=n.std(tsmT)
    mvars=("mtmT","dtmT","mttmT","dttmT","mtsmT","dtsmT")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def medidasPOS_(list_of_list_of_sentences_tokenized):
    # [i["tokens_sentences"] for i in sent_measures]
    return [medidasPOS(i) for i in list_of_list_of_sentences_tokenized]
def medidasPOS(sentences_tokenized):
    """Measures of POS tags

    Receives a sequence of sentences,
    each as a sequence of tokens.
    Returns a set measures of POS tags,
    and the tagged sentences.

    Convention:
    VERB - verbs (all tenses and modes)
    NOUN - nouns (common and proper)
    PRON - pronouns 
    ADJ - adjectives
    ADV - adverbs
    ADP - adpositions (prepositions and postpositions)
    CONJ - conjunctions
    DET - determiners
    NUM - cardinal numbers
    PRT - particles or other function words
    X - other: foreign words, typos, abbreviations
    . - punctuation
    
    See "A Universal Part-of-Speech Tagset"
    by Slav Petrov, Dipanjan Das and Ryan McDonald
    for more details:
        http://arxiv.org/abs/1104.2086"""

    tags=brill_tagger.tag_sents(sentences_tokenized)
    tags_=[item for sublist in tags for item in sublist]
    tags__=[i[1] for i in tags_ if i[0].lower() in WL_]
    htags=c.Counter(tags__)
    htags__=c.OrderedDict()
    if htags:
       	factor=100.0/sum(htags.values())
        htags_={}
        for i in htags.keys(): htags_[i]=htags[i]*factor    
        htags__=c.OrderedDict(sorted(htags_.items(), key=lambda x: -x[1]))
    mvars=("htags__","tags")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict

def S(acounter):
    return sorted(acounter.items(),key=lambda x: -x[1])
def auxWnTb(tt,level,tabfname,wn_dict_list):
    tt_=[S(i) for i in tt]
    labels=[i[0] for i in tt_[0][:12]]
    if labels:
        wms_=[[tt[i][j] for i in range(4)] for j in labels]
        labels=[i.replace("_","\_") for i in labels]
        if level=="root":
            caption=r"""Counts for the most incident synsets at the semantic roots in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, {{\bf h.}} for hubs). Yes.""".format(level)
        else:
            caption=r"""Counts for the most incident synsets {} step from the semantic roots in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, {{\bf h.}} for hubs).""".format(level)
        # normalizar este data com relação às colunas
        B.me.append(wms_)
        B.tt_.append(tt_)
        B.tt.append(tt)
        data=n.array(wms_)
        data=100*data/data.sum(axis=0)
        data=data[:12]
        data=n.vstack((data,data.sum(axis=0)))
        labels+=[r"{{\bf total}}"]
        g.lTable(labels,labelsh,data,caption,tabfname,"textGeral_")
        ME(tabfname[:-4],"\\bf",[(0,i) for i in range(1,5)])
        DL(tabfname[:-4]+"_",[1,-3],[1])
    else:
        print(tabfname.split("/")[-1], "No labels:",labels,
                "\nProbably no hypernyms:",
              len(wn_dict_list[0]["top_hypernyms"]))


def makeWordnetTable2a(wn_dict_list, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="wnInline2a.tex"):
    """Table about the most incident roots"""
    t0=[c.Counter([i[0].name() for i in j["top_hypernyms"]]) for j in wn_dict_list]
    auxWnTb(t0,"root",table_dir+fname,wn_dict_list)
def makeWordnetTable2b(wn_dict_list, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="wnInline2b.tex"):
    """Table about the most incident roots"""
    t1=[c.Counter([i[1].name() for i in j["top_hypernyms"] if len(i)>1]) for j in wn_dict_list]
    #auxWnTb(labels,labelsh,data,level,tabfname)
    auxWnTb(t1,"one",table_dir+fname,wn_dict_list)
def makeWordnetTable2c(wn_dict_list, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="wnInline2c.tex"):
    """Table about the most incident roots"""
    t2=[c.Counter([i[2].name() for i in j["top_hypernyms"] if len(i)>2]) for j in wn_dict_list]
    auxWnTb(t2,"two",table_dir+fname,wn_dict_list)
def makeWordnetTable2d(wn_dict_list, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="wnInline2d.tex"):
    """Table about the most incident roots"""
    t3=[c.Counter([i[3].name() for i in j["top_hypernyms"] if len(i)>3]) for j in wn_dict_list]
    auxWnTb(t3,"three",table_dir+fname,wn_dict_list)
def makeWordnetPOSTable(wn_dict_list, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="wnPOSInline.tex",tag=None):
    wms=wn_dict_list
    labels=["N","ADJ","VERB","ADV","POS","POS!"]
    data=[[wms[i]["ftags"][j] for i in range(4)] for j in range(4)]
    # incluir % com relação aas palavras totais etiquetadas
    # variaveis [posok WL_ e posnok
    data+=[[100*len(wms[i]["posok"])/len(wms[i]["WT_"]) for i in range(4)]]
    data+=[[100*(len(wms[i]["posok"])/(len(wms[i]["posok"])+len(wms[i]["posnok"]))) for i in range(4)]]
    caption=r"""Percentage of synsets with each of the POS tags used by Wordnet. The last lines give the percentage of words considered from all of the tokens (POS) and from the words with synset (POS!). The tokens not considered are punctuations, unrecognized words, words without synsets, stopwords and words for which Wordnet has no synset  tagged with POS tags . Values for each Erd\"os sectors are in the columns {{\bf p.}} for periphery, {{\bf i.}} for intermediary, {{\bf h.}} for hubs."""
    labelsh=("","g.","p.","i.","h.")
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels,labelsh,data,caption,fname_,"textGeral_")
    ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)])
    DL(fname_[:-4]+"_",[1,-4],[1])
def medidasWordnet2_POS(wn_measures,poss=("n","as","v","r")):
    wn_measures2={}
    for pos in poss:
        wn_measures2[pos]=g.textUtils.medidasWordnet2_(wn_measures,pos)
    return wn_measures2
def makeWordnetTables2_POS(wn_dict_pos, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="wnPOSInline2",poss=("n","as","v","r"),tag=None):
    TDIR=table_dir
    for pos in poss:
        wn_measures2=wn_dict_pos[pos]
        g.textUtils.makeWordnetTable(  wn_measures2,TDIR  ,fname="{}-{}-{}tag.tex". format(fname,pos,tag)) # medias e desvios das incidencias dos atributos
        g.textUtils.makeWordnetTable2a(wn_measures2,TDIR,  fname="{}a-{}-{}tag.tex".format(fname,pos,tag)) # contagem dos synsets raiz
        g.textUtils.makeWordnetTable2b(wn_measures2,TDIR,  fname="{}b-{}-{}tag.tex".format(fname,pos,tag)) # contagem dos synsets raiz
        g.textUtils.makeWordnetTable2c(wn_measures2,TDIR,  fname="{}c-{}-{}tag.tex".format(fname,pos,tag)) # contagem dos synsets raiz
        g.textUtils.makeWordnetTable2d(wn_measures2,TDIR,  fname="{}d-{}-{}tag.tex".format(fname,pos,tag)) # contagem dos synsets raiz
    # make one file from all 20 (max) tables
    names="{}-{}_.tex","{}a-{}_.tex","{}b-{}_.tex","{}c-{}_.tex","{}d-{}_.tex"
    tx=""
    for pos in poss:
        tx+="\n\n% POS -> "+pos
        for name in names:
            name_=TDIR+name.format(fname,pos,tag)
            if os.path.isfile(name_):
                tx+="\n% fname -> "+name_+"\n"
                tx+=open(name_).read()
    g.writeTex(tx,TDIR+fname+".tex")
def makeWordnetTable(wn_dict_list, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="wnInline.tex"):
    wms=wn_dict_list
    mvars=("mmind","dmind",
           "mmaxd","dmaxd",
           "mnhol_","dnhol_",
           "mnmer_","dnmer_",
           "mndomains","dndomains",
           "mnsimilar","dnsimilar",
           "mnverb_groups","dnverb_groups",
           "mnlemmas","dnlemmas",
           "mnentailments","dnentailments",
           "mnhypo_","dnhypo_",
           "mnhyper_","dnhyper_",
           )

    wms_=[[wms[j][i] for j in range(4)] for i in mvars]
    labelsh=("","g.","p.","i.","h.")
    labels=("$\mu(min\,depth)$","$\sigma(min\,depth)$",
            "$\mu(max\,depth)$",r"$\sigma(max\,depth)$",
            "$\mu(holonyms)$",    "$\sigma(holonyms)$",
            "$\mu(meronyms)$",    "$\sigma(meronyms)$",
            "$\mu(domains)$",     "$\sigma(domains)$",
            "$\mu(similar)$",     "$\sigma(similar)$",
            "$\mu(verb\,groups)$","$\sigma(verb\,groups)$",
            "$\mu(lemmas)$",      "$\sigma(lemmas)$",
            "$\mu(entailments)$", "$\sigma(entailments)$",
            "$\mu(hyponyms)$",    "$\sigma(hyponyms)$",
            "$\mu(hypernyms)$",   "$\sigma(hypernyms)$",
        )
    caption=r"""Measures of wordnet features in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, {{\bf h.}} for hubs)."""
    data=wms_
    g.lTable(labels,labelsh,data,caption,table_dir+fname,"textGeral_")
    ME(table_dir+fname[:-4],"\\bf",[(0,i) for i in range(1,5)])
    DL(table_dir+fname[:-4]+"_",[1],[1],[2,4,6,8,10,12,14,16,18,20,22])



def makePOSTable(posMensagens_dict, table_dir="/home/r/repos/artigoTextoNasRedes/tables/",fname="posInline.tex",tag=None):
    pms=posMensagens_dict
#    pms_=[list(i["htags__"].items()) for i in pms]
    #mvars=[list(i["htags__"].keys()) for i in pms]
    #mvars=list(pms[0]["htags__"].keys())
    mvars=['NOUN', 'X', 'ADP', 'DET', 'VERB', 'ADJ', 'ADV', 'PRT', 'PRON', 'NUM', 'CONJ',"."]
    pms__=[[pms[j]["htags__"][i] if (i in pms[j]["htags__"].keys()) else 0 for j in range(4)] for i in mvars]
    labelsh=("","g.","p.","i.","h.")
    labels=mvars[:-1]+["PUNC"]
    caption=r"""POS tags in each Erd\"os sector ({{\bf p.}} for periphery, {{\bf i.}} for intermediary, {{\bf h.}} for hubs).
    Universal POS tags~\cite{{petrov}}:
    VERB - verbs (all tenses and modes);
    NOUN - nouns (common and proper);
    PRON - pronouns;
    ADJ - adjectives;
    ADV - adverbs;
    ADP - adpositions (prepositions and postpositions);
    CONJ - conjunctions;
    DET - determiners;
    NUM - cardinal numbers;
    PRT - particles or other function words;
    X - other: foreign words, typos, abbreviations;
    PUNCT - punctuation.
"""
    #data=list(map(list, zip(*tms_)))
    data=pms__
    #nmsgs=data[0]
    #nmsgs_=perc_(nmsgs)
    #data=n.array(data[1:])
    #data=n.vstack((nmsgs,nmsgs_,data))
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels,labelsh,data,caption,fname_,"textGeral_")
    ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,5)])
    DL(fname_[:-4]+"_",[1],[1],[2,4,7,9,10,11,12])


def filtro(wt_):
    # faz separação dos tokens para analise com wordnet
    sword_sem_synset=[]
    sword_com_synset=[]
    word_com_synset=[]
    word_sem_synset=[]
    pontuacao=[]
    token_exotico=[]
    for wt in wt_:
        ss=wn.synsets(wt[0])
        if ss:
            if wt[0] in stopwords:
                sword_com_synset.append(wt)
            else:
                word_com_synset.append((wt[0],wt[1],ss))
        #elif wt[0] in puncts:
        elif sum([tt in puncts for tt in wt[0]])==len(wt[0]):
            pontuacao.append(wt)
        elif wt[0] in stopwords:
            sword_sem_synset.append(wt)
        elif wt[0] in WL_:
            word_sem_synset.append(wt)
        else:
            token_exotico.append(wt)
    mvars=("sword_sem_synset","sword_com_synset","word_com_synset","word_sem_synset","pontuacao","token_exotico")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def traduzPOS(astring):
    if astring in ("NOUN","NNS","NN","NUM"):
        return wn.NOUN
    elif astring in ("VERB","VBG"):
        return wn.VERB
    elif astring in ("ADJ","JJ","ADP"):
        return wn.ADJ+wn.ADJ_SAT
    elif astring in ("ADV","RB","PRT"):
        return wn.ADV
    else:
        return "NOPOS"
        
def medidasWordnet_(list_words_with_pos_tags):
    return [medidasWordnet(i) for i in list_words_with_pos_tags]
def medidasWordnet2_(list_wn_stuff,pos):
    return [medidasWordnet2(i,pos) for i in list_wn_stuff]
def medidasWordnet(words_with_pos_tags):
    WT=words_with_pos_tags
    WT_=[(i[0].lower(),i[1]) for j in WT for i in j]
    wlists=filtro(WT_)
    wl=wlists["word_com_synset"]
    posok=[]
    posnok=[]
    for ww in wl:
        pos = traduzPOS(ww[1])
        ss=ww[2]
        # procura nos nomes dos synsets o pos e numeracao mais baixa
        poss=[i.pos() for i in ss]
        fposs=[pp in pos for pp in poss]
        if sum(fposs):
            tindex=fposs.index(True)
            posok.append((ww[0],ss[tindex]))
        else:
            posnok.append(ww)
    # estatísticas sobre posok
    # quais as tags?
    posok_=[i[1].pos() for i in posok]
    ftags_=[100*posok_.count(i)/len(posok_) for i in ('n', 's','a', 'r', 'v')]
    ftags=ftags_[0:2]+ftags_[3:]
    ftags[1]+=ftags_[2]
    mvars=("WT_","wlists","posok","posnok","ftags")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def medidasWordnet2(wndict,pos=None):
    """pos={'r', 'as', 'n', 'v'}"""
    sss=wndict["posok"]
    if pos:
        sss_=[i[1] for i in sss if i[1].pos() in pos]
    else:
        sss_=[i[1] for i in sss]
    hyperpaths=[i.hypernym_paths() for i in sss_]
    top_hypernyms=[i[0][:4] for i in hyperpaths] # fazer histograma por camada
    lexnames=[i.lexname().split(".")[-1] for i in sss_] # rever

    mhol=[len(i.member_holonyms()) for i in sss_]
    phol=[len(i.part_holonyms()) for i in sss_]
    shol=[len(i.substance_holonyms()) for i in sss_]
    nhol_=[mhol[i]+phol[i]+shol[i] for i in range(len(sss_))] ###

    mmer=[len(i.member_meronyms()) for i in sss_] #
    pmer=[len(i.part_meronyms()) for i in sss_]
    smer=[len(i.substance_meronyms()) for i in sss_]
    nmer_=[mmer[i]+pmer[i]+smer[i] for i in range(len(sss_))] ###

    nlemmas=[len(i.lemmas()) for i in sss_] ###
    nhyperpaths=[len(i) for i in hyperpaths]
    shyperpaths=[len(i) for j in hyperpaths for i in j]

    nentailments=[len(i.entailments()) for i in sss_]

    nhypernyms=[len(i.hypernyms()) for i in sss_]
    nihypernyms=[len(i.instance_hypernyms()) for i in sss_]
    nhyper_=[nhypernyms[i]+nihypernyms[i] for i in range(len(sss_))]

    nhypo=[len(i.hyponyms()) for i in sss_] ###
    nihypo=[len(i.instance_hyponyms()) for i in sss_]
    nhypo_=[nhypo[i]+nihypo[i] for i in range(len(sss_))]

    maxd=[i.max_depth() for i in sss_] ###
    mind=[i.min_depth() for i in sss_] ###

    nregion_domains=[len(i.region_domains()) for i in sss_] #
    ntopic_domains= [len(i.topic_domains())  for i in sss_]
    nusage_domains= [len(i.usage_domains())  for i in sss_]
    ndomains=[nregion_domains[i]+ntopic_domains[i]+nusage_domains[i]
            for i in range(len(sss_))] ###

    nsimilar=[    len(i.similar_tos()) for i in sss_]
    nverb_groups=[len(i.verb_groups()) for i in sss_]

    mvars=list(locals().keys()); mvars.remove("wndict")
    mvars_=mvars[:]
    mvars_.remove("sss_");       mvars_.remove("sss");
    mvars_.remove("top_hypernyms")
    mvars_.remove("pos")
    mvars_.remove("hyperpaths"); mvars_.remove("lexnames")
    vdict={}
    #mvars=("nmero_part",)
    locals_=locals()
    for mvar in mvars:
        if mvar not in mvars_:
            vdict[mvar] = locals_[mvar]
        else:
            vdict["m"+mvar]=n.mean(locals_[mvar])
            vdict["d"+mvar]=n.std(locals_[mvar])
    return vdict
def medidasParticipante(dict_auth_text):
    medidas_autor={}
    for author in dict_auth_text:
        text=dict_auth_text[author]
        if text:
            text_,ncontract=R(text)
            medidas=medidasSentencas(text_)
            medidas2=medidasPOS(medidas["tokens_sentences"])
            medidas.update(medidas2)
            medidas_autor[author]=medidas
    return medidas_autor


def medidasPCA2_(ds,nm,authors_lists=None):
    mall=medidasPCA2(ds,nm)
    return [mall]+[medidasPCA2(ds,nm,authors) for authors in authors_lists]
def medidasPCA2(ds,nm,authors=None):
    textosP= textosParticipante(ds,authors)
    medidasP=medidasParticipante(textosP)
    medidas_autor=g.textUtils.medidasPCA(medidasP,nm)

    vkeys=["clustering","degree","strength","Mpuncts_sents","Spuncts_sents","Mknownw_sents","Sknownw_sents","Mstopw_sents","Sstopw_sents"]
    pca=g.textUtils.tPCA(medidas_autor,vkeys)
    vlabels=[r"$cc$",r"$d$",r"$s$",r"$\mu_S(p)$",r"$\sigma_S(p)$",r"$\mu_S(kw)$",r"$\sigma_S(kw)$",r"$\mu_S(sw)$",r"$\sigma_S(sw)$"]
    mvars=("vlabels","pca","vkeys","medidas_autor","medidasP","textosP")
    vdict={}
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    return vdict
def medidasPCA(medidas_participante_dict,network_measures):
    nm,mp=network_measures,medidas_participante_dict
    for author in mp:
        mp[author]["degree"]=nm.degrees[author]
        mp[author]["strength"]=nm.strengths[author]
        mp[author]["clustering"]=nm.clusterings[author]
    return mp
def textosParticipante(ds,authors=None):
    texts={}
    if not authors:
        authors=ds.author_messages
    for author in authors:
        texts[author]=""
        for msg in ds.author_messages[author]:
            msgid=msg[0]
            text=ds.messages[msgid][-1]
            texts[author]+=text
            B.LANG+=[langid.classify(text)]
    return texts
def makePCATable_(medidas_pca,table_dir,fname="pcaInline.tex",tag=None):
    vecs=[i["pca"].feature_vec_.real for i in medidas_pca]
    vals=[i["pca"].eig_values_.real for i in medidas_pca]
    labelsh=[""]+["PC{}".format(i+1) for i in range(vecs[0].shape[1])]
    labels=medidas_pca[0]["vlabels"]
    labels=labels+[r"$\lambda$"]
    data=[]
    for secn in range(len(vecs[0])):
        for vec in vecs:
            data.append(vec[secn])
    caption="PCA formation"
    data=n.vstack(data+[val[:vecs[0].shape[1]] for val in vals])
    labels_=[(i,"","","") for i in labels]
    labels__=[i for j in labels_ for i in j]
    labels__[1:4]=["(p.)","(i.)","(h.)"]
    fname_=mkName(table_dir,fname,tag)
    g.lTable(labels__,labelsh,data,caption,fname_,"textPCA")
    ME(fname_[:-4],"\\bf",[(0,i) for i in range(1,6)]+[(i,0) for i in range(1,11)])
    DL(fname_[:-4]+"_",[1,-6],[1],[2,3,4,
                                         6,7,8,
                                         10,11,12,
                                         14,15,16,
                                         18,19,20,
                                         22,23,24,
                                         26,27,28,
                                         30,31,32,
                                         34,35,36,
                                         38,39,40])

def makePCATable(vecs,vals,labs,table_dir,fname="pcaInline.tex"):
    labelsh=[""]+["PC{}".format(i+1) for i in range(vecs.shape[1])]
    labels=labs+[r"$\lambda$"]
    data=n.vstack((vecs,vals[:vecs.shape[1]]))
    caption="PCA formation"
    g.lTable(labels,labelsh,data,caption,table_dir+fname,"textGeral_")
    ME(table_dir+fname[:-4],"\\bf",[(0,i) for i in range(1,6)]+[(i,0) for i in range(1,11)])
    DL(table_dir+fname[:-4]+"_",[1,-3],[1],[2,3,5,7,8])
def tPCA(medidas,keys):
    data=[]
    for author in medidas:
        data+=[[]]
        for key in keys:
            data[-1]+=[medidas[author][key]]
    data_=n.array(data)
    data__=data_.T
    return g.pca.PCA(data__,final_dimensions=5)


from sklearn.feature_extraction.text import TfidfVectorizer
def tfIdf(texts):
    """Returns distance matrix for the texts"""
    vect = TfidfVectorizer(min_df=1)
    tfidf = vect.fit_transform([tt.lower() for tt in texts])
    aa=(tfidf * tfidf.T).A
    return aa
def kolmogorovSmirnovDistance(seq1,seq2,bins=300):
    """Calculate distance between histograms
    
    Adapted from the Kolmogorov-Smirnov test"""
    amin=min(min(seq1),min(seq2))
    amax=max(max(seq1),max(seq2))
    bins=n.linspace(amin,amax,bins+1,endpoint=True)
    h1=n.histogram(seq1,bins,density=True)[0]
    h2=n.histogram(seq2,bins,density=True)[0]
    space=bins[1]-bins[0]
    cs1=n.cumsum(h1*space)
    cs2=n.cumsum(h2*space)

    dc=n.abs(cs1-cs2)
    Dnn=max(dc)
    n1=len(seq1)
    n2=len(seq2)
    fact=((n1+n2)/(n1*n2))**0.5
    calpha=Dnn/fact
    return calpha
def uniteTables3(TDIR,tag):
    """junta cada POS tag da wn em uma tabelona"""
    tt="wnPOSInline2a","wnPOSInline2b","wnPOSInline2c","wnPOSInline2d",
    fnames=[]
    for pos in ("n","as","v","r"):
    #for pos in ("n",):
        fname=TDIR+"wnPOSInline2-{}-{}".format(pos,tag)
        fnames=[]
        for ttt in tt:
            fnames+=[TDIR+ttt+"-{}-{}tag_".format(pos,tag)]
        if os.path.isfile(fnames[1]+".tex"):
            g.tableHelpers.vstackTables_(fnames[0],fnames[1],fname)
        else:
            shutil.copyfile(fnames[0]+".tex",fname+".tex")
        if os.path.isfile(fnames[2]+".tex"):
            g.tableHelpers.vstackTables_(fname,fnames[2],fname)
        if os.path.isfile(fnames[3]+".tex"):
            g.tableHelpers.vstackTables_(fname,fnames[3],fname)
def uniteTables2(TDIR,tag):
    foo=TDIR+"posMerged{}".format(tag)
    g.tableHelpers.vstackTables_(TDIR+"posInline{}_".format(tag),
            TDIR+"wnPOSInline{}_".format(tag),foo)
def uniteTables(TDIR,tag):
    t1="geral"#"geralInline0_"
    t2="chars"
    t3="tokensMerged"
    t4="sentences"
    t5="messages"
    def makeN(ss):
        if ss==t3:
            return ss+"Inline{}".format(tag)
        return ss+"Inline{}_".format(tag)
    tt=[TDIR+makeN(i) for i in (t1,t2,t3,t4,t5)]
    foo=TDIR+"mergedA{}".format(tag)
    g.tableHelpers.vstackTables(tt[0],tt[1],foo)
    g.tableHelpers.vstackTables(foo,tt[2],foo)
    g.tableHelpers.vstackTables(foo,tt[3],foo)
    g.tableHelpers.vstackTables(foo,tt[4],foo)


def makeTables_(lids,TOTAL,TDIR,FDIR,tags=None,offset=0,start_from=0,basedir="~/.gmane3/"):
    # if not tags:
    #     tags=[str(i) for i in range(len(lids))]
    # for lid,tag in zip(lids,tags):
    #     es=g.EmailStructures(lid,TOTAL,offset=offset,basedir=basedir)
    #     if sum([len(i)>4 for i in es.structs[-1].sectorialized_agents__])<3:
    #         B.degen.append(lid)
    #         continue
    #     isenglish = makeTable(lid,es,TOTAL,TDIR,FDIR,tag)
    #     if isenglish == 'nonenglish':
    #         B.nonenglish.append(lid)
    B.degen=[]
    B.nonenglish=[]
    tag = 0
    for lid in lids:
        es=g.EmailStructures(lid,TOTAL,offset=offset,basedir=basedir)
        if sum([len(i)>4 for i in es.structs[-1].sectorialized_agents__])<3:
            B.degen.append(lid)
            print("------- >       > Degenerated structure <")
        else:
            isenglish = makeTable(lid,es,TOTAL,TDIR,FDIR,tag)
            if isenglish == 'nonenglish':
                B.nonenglish.append(lid)
            else:
                tag += 1
    tags = list(range(tag))
    lids_ = [i for i in lids if i not in B.degen and i not in
            B.nonenglish]
    labelsh = ('tag', 'gmane id')
    labels = [str(i) for i in tags]
    data = [[i] for i in lids_]
    caption = 'Numerical tags with respective list ids used throughout tables in this supporting information document.'
    fname_ = TDIR+'labelsIDs.tex'
    g.tableHelpers.lTable(labels,labelsh,data,caption,fname_,"strings")

def makeTable(lid,es,TOTAL,TDIR,FDIR,tag,offset=0):
    #TDIR="/home/r/repos/artigoTextoNasRedes/tables/"
    #TDIRf="/home/r/repos/artigoTextoNasRedes/figs/"
    ds=es.structs[1]
    timest=es.structs[2]
    pr=es.structs[-1]
    nm=es.structs[4]
    B.LANG=[]
    B.tag=tag

    ts,ncontractions,msg_ids=g.textUtils.makeText_(ds,pr); check("make text")
    B.LANG+=[langid.classify(" ".join(ts))]
    if B.LANG[-1][0] != 'en':
        print("NON ENGLISH LIST", B.LANG[-1])
        return 'nonenglish'
    else:
        print("IS ENGLISH!!!! <<<<<<<<<=================>>>>>>>>>>> !!!!!!!")

    gmeasures=g.generalMeasures(ds,pr,timest)
    g.makeGeneralTable(gmeasures,TDIR,tag=tag)

    char_measures=g.textUtils.medidasLetras_(ts); check("medidas letras")
    g.textUtils.makeCharTable(char_measures,TDIR,tag=tag)
    
    tok_measures=g.textUtils.medidasTokens__(ts,ncontractions); check("medidas tokens")
    g.textUtils.makeTokensTable(tok_measures,TDIR,tag=tag)
    g.textUtils.makeTokenSizesTable(tok_measures,TDIR,tag=tag)
    g.tableHelpers.vstackTables(TDIR+"tokensInline{}_".format(tag),TDIR+"tokenSizesInline{}_".format(tag),TDIR+"tokensMergedInline{}".format(tag))
    
    sent_measures=g.textUtils.medidasSentencas_(ts); check("medidas sentenças")
    g.textUtils.makeSentencesTable(sent_measures,TDIR,tag=tag)
    
    msg_measures=g.textUtils.medidasMensagens_(ds,msg_ids); check("medidas mensagens")
    g.textUtils.makeMessagesTable(msg_measures,TDIR,tag=tag)

    g.textUtils.uniteTables(TDIR,tag)
    
    pos_measures=g.textUtils.medidasPOS_([i["tokens_sentences"] for i in sent_measures]); check("medidas POS")
    g.textUtils.makePOSTable(pos_measures,TDIR,tag=tag)
    
    wn_measures=g.textUtils.medidasWordnet_([i["tags"] for i in pos_measures]); check("medidas wordnet")
    g.textUtils.makeWordnetPOSTable(wn_measures,TDIR ,tag=tag) # medias e desvios das incidencias dos atributos

    g.textUtils.uniteTables2(TDIR,tag)
    
    wn_measures2_pos=g.textUtils.medidasWordnet2_POS(wn_measures); check("medidas wordnet 2")
    g.textUtils.makeWordnetTables2_POS(wn_measures2_pos,TDIR,tag=tag) # escreve arquivo com todas as 5 tabelas para cada pos

    g.textUtils.uniteTables3(TDIR,tag)

    sinais=g.textUtils.medidasSinais_(ts); check("medidas sinais")
    dists=g.textUtils.ksAll(sinais,mkeys=["lens_tok","lens_word","lens_sent"]); check("ks sinais")
    g.textUtils.makeKSTables(dists,TDIR,
            fnames=("ksTokens","ksWords","ksSents"),
            tags=("size of tokens","size of known words","size of sentences"),tag=tag)

    sinais2=g.textUtils.medidasSinais2_(pos_measures,msg_measures); check("medidas sinais 2")
    dists2=g.textUtils.ksAll(sinais2,mkeys=["adj","sub","pun","verb","chars"]); check("ks sinais 2")
    g.textUtils.makeKSTables(dists2,TDIR,
            fnames=("ksAdjs","ksSubs","ksPuns","ksVerbs","ksChars"),
            tags=("use of adjectives on sentences","use of substantives on sentences","use of punctuations on sentences","use of verbs in each 100 tokens","use of number of characters in messages"),tag=tag)

    # correlação pierson e spearman (tem necessidade das duas?)
    medidas_pca=g.textUtils.medidasPCA2_(ds,nm,pr.sectorialized_agents__); check("medidas pca") # retorna medidas para plotar e tabelas
    g.textUtils.makeCorrelationTable_(medidas_pca,TDIR,"correlationInline.tex",tag=tag)
    g.textUtils.makePCATable_(medidas_pca,TDIR,tag=tag)
    # medidas_pca[0]["pca"].plot("plot_pca-{}.png".format(tag),pr,labels="sym",tdir=FDIR)
    es.structs=es.structs[1:]
    ftags=[i["ftags"] for i in wn_measures]
    LANG=B.LANG
    mvars=("es","gmeasures","ts","ncontractions","msg_ids",
            "char_measures","tok_measures","sent_measures",
            "msg_measures","pos_measures","ftags",
            "sinais","sinais2","dists2","medidas_pca","LANG","tag")
    vdict={}; check("antes da escrita do pickle")
    for mvar in mvars:
        vdict[mvar] = locals()[mvar]
    pDump(vdict,TDIR+"vdict-{}.pickle".format(tag))
    check("escrito pickle, {}, {}".format(lid, TDIR))
    del B.tag

    return 0



import pylab as p, matplotlib, os, pickle
from .networkPartitioning import *

class EvolutionTimelines:
    def __init__(self,tdir="./evolution/",label="gmaneID",draw=True):
        self.tdir=tdir
        self.label=label
        self.getMeasures()
        if draw:
            self.drawTimelines()
    def getOverallMeasures(self):
        filenames=os.listdir(self.tdir)
        filenames_=[i for i in filenames if i.endswith(".pickle")]
        filename=[i for i in filenames_ if "overall" in i][0]
        with open(self.tdir+filename,"rb") as f:
                self.overall=pickle.load(f)
    def getMeasures(self):
        self.getOverallMeasures()
        filenames=os.listdir(self.tdir)
        filenames_=[i for i in filenames if i.endswith(".pickle")]
        filenames_=[i for i in filenames_ if i.startswith("im")]
        filenames_.sort()
        agents={"s":[],"is":[],"os":[],
                "d":[],"id":[],"od":[],
                "exc":[],"excc":[],"exce":[],
                "inc":[],"incc":[],"ince":[],
                "nm":[],"totals":[]}
        for filename in filenames_:
            with open(self.tdir+filename,"rb") as f:
                data=pickle.load(f)
                nm=data["nm"]
                agents["nm"].append(nm)
                n_agents=nm.N
                agents["totals"].append(n_agents)
                minimum_incidence=data["np"].minimum_incidence
                agents["s"].append(data["np"].sectorialized_agents__)
                agents["is"].append(
                        NetworkPartitioning(nm,minimum_incidence=minimum_incidence,metric="is").sectorialized_agents__ )
                agents["os"].append(
                        NetworkPartitioning(nm,minimum_incidence=minimum_incidence,metric="os").sectorialized_agents__ )
                agents["d"].append(
                        NetworkPartitioning(nm,minimum_incidence=minimum_incidence,metric="d").sectorialized_agents__ )
                agents["id"].append(
                        NetworkPartitioning(nm,minimum_incidence=minimum_incidence,metric="id").sectorialized_agents__ )
                agents["od"].append(
                        NetworkPartitioning(nm,minimum_incidence=minimum_incidence,metric="od").sectorialized_agents__ )
                # pegar os agentes da rede em
                # todos os outros critérios de particionamento simples:
                # d, id, od, is, os
                # e mandar eles para uma função que já calcula
                # os particionamentos compostos
                compound=compoundPartitioning(agents)
                agents["exc"].append(compound["exc"])
                agents["excc"].append(compound["excc"])
                agents["exce"].append(compound["exce"])
                agents["inc"].append(compound["inc"])
                agents["incc"].append(compound["incc"])
                agents["ince"].append(compound["ince"])
        self.agents=agents
    def plotFracs(self,ttype,subplot,ate,step_size):
        p.subplot(subplot)
        p.title(ttype)
        if ttype == "degree": ttype_="d"
        if ttype == "strength": ttype_="s"
        if ttype == "in-degree": 
            ttype_="id"
            p.ylabel(r"fraction of nodes in each section $\rightarrow$")
        if ttype == "out-degree": ttype_="od"
        if ttype == "in-strength": 
            ttype_="is"
            p.ylabel(r"fraction of nodes in each section $\rightarrow$")
        if ttype == "out-strength": ttype_="os"
        if ttype == "exclusivist":
            ttype_="exc"
            not_classified=list([1-sum(self.fractionLengths(i,total)) 
                for i,total in zip(self.agents[ttype_],self.agents["totals"])])
            p.plot(list(range(0,ate,step_size))[:-1],not_classified,"k-.x")
        if ttype == "inclusivist":
            ttype_="inc"
            super_classified=list([sum(self.fractionLengths(i,total))-1 for i,total in zip(self.agents[ttype_],self.agents["totals"])])
            p.plot(list(range(0,ate,step_size))[:-1],super_classified,"k-.x")
        if ttype == "exclusivist cascade": 
            ttype_="excc"
            p.ylabel(r"fraction of nodes in each section $\rightarrow$")
        if ttype == "inclusivist cascade": 
            ttype_="incc"
            p.ylabel(r"fraction of nodes in each section $\rightarrow$")
        if ttype == "exclusivist externals": 
            ttype_="exce"
            p.xlabel(r"messages $\rightarrow$")
        if ttype == "inclusivist externals": 
            ttype_="ince"
            p.xlabel(r"messages $\rightarrow$")
        fractions=[self.fractionLengths(i,total) for i,total in zip(self.agents[ttype_],self.agents["totals"])]
        hubs_fractions=[i[2] for i in fractions]
        intermediary_fractions=[i[1] for i in fractions]
        periphery_fractions=[i[0] for i in fractions]
        print(list(range(0,ate,step_size)),periphery_fractions)
        print(len(list(range(0,ate,step_size))),
              len(periphery_fractions))
        #p.plot(list(range(0,ate,step_size))[:-1],periphery_fractions,   "b")
        #p.plot(list(range(0,ate,step_size))[:-1],intermediary_fractions,"g")
        #p.plot(list(range(0,ate,step_size))[:-1],hubs_fractions,        "r")
        p.plot(list(range(0,ate,step_size))[:-1],periphery_fractions,   "ko-",ms=8,alpha=.7,label="periphery")
        p.plot(list(range(0,ate,step_size))[:-1],intermediary_fractions,"k^-",ms=8,alpha=.7,label="intermediary")
        p.plot(list(range(0,ate,step_size))[:-1],hubs_fractions,        "k*-",ms=10,alpha=.7,label="hubs")
        p.ylim(0,1.)
        p.xlim(-5,ate-1700+5)
    def plotMeasure(self,title,subplot,ate,step_size):
        if subplot=="5,2,10":
            p.subplot(5,2,10)
        else:
            p.subplot(subplot)
        p.title(title)
        if title=="total weight":
            measures=[sum([i[2]["weight"] for i in nm.edges])
                    for nm in self.agents["nm"]]
            p.ylabel(r"$\sum s_i \;\;\rightarrow$")
        elif title=="number of edges":
            measures=[i.E for i in self.agents["nm"]]
            #p.ylabel(r"$\mathfrak{z} \rightarrow$",fontsize=20)
            p.ylabel(r"$z \rightarrow$",fontsize=20)
        elif title=="number of vertices":
            measures=[i.N for i in self.agents["nm"]]
            p.xlabel(r"messages $\rightarrow$")
            p.ylabel(r"N $\rightarrow$")
        elif title=="center, periphery and discon.":
            center=[len(i.center) for i in self.agents["nm"]]
            p.plot(range(0,ate,step_size),center,"b")
            periphery=[len(i.periphery) for i in self.agents["nm"]]
            p.plot(range(0,ate,step_size),periphery,"k",linestyle="dashed")
            periphery_=[len(i.periphery_) for i in self.agents["nm"]]
            measures=[i+j for i,j in zip(periphery,periphery_)]
            p.xlabel(r"messages $\rightarrow$")
            p.ylabel("number of\nnodes"+r"$\rightarrow$")

        p.plot(range(0,ate,step_size),measures,"k")
        p.ylim(min(measures)*.99,max(measures)*1.01)
        p.xlim(-5,ate+5)

    def plotSingles(self):
        #p.clf()
        #fig = matplotlib.pyplot.gcf()
        #fig.set_size_inches(10.5,3.4) ###
        p.figure(figsize=(10.,4.))
        #ate=self.overall[1][0].n_messages-self.overall[0]["window_size"]
        ate=self.overall[1][0].n_messages
        step_size=self.overall[0]["step_size"]
        p.suptitle(r"Empirical fractions of participants in each of the Erdös sectors")
        #p.suptitle((r"Fraction of participants in each Erdös Sector. Window: %i messages."+"\nPlacement resolution: %i messages. %s") % (self.overall[0]["window_size"],step_size,self.label))

        self.plotFracs("degree",     "221",ate,step_size)
        p.ylabel(r"$\overline{e_{1,x}} \;\rightarrow$",fontsize=20)
        self.plotFracs("strength",   "223",ate,step_size)
        p.ylabel(r"$\overline{e_{4,x}} \;\rightarrow$",fontsize=20)
        p.xlabel(r"messages $\;\rightarrow$",fontsize=15)
        self.plotFracs("exclusivist","222",ate,step_size)
        p.ylabel(r"$\overline{c_{1,x}} \;\rightarrow$",fontsize=20)
        self.plotFracs("inclusivist","224",ate,step_size)
        p.ylabel(r"$\overline{c_{2,x}} \;\rightarrow$",fontsize=20)
        p.xlabel(r"messages $\;\rightarrow$",fontsize=15)
        #fig.set_size_inches(5.5,16.4) ###

        p.subplots_adjust(left=0.08,bottom=0.13,right=0.99,top=0.79,wspace=0.23,hspace=0.47)

        p.plot([],[],        "k-.x",label="without sector or ambiguous")
        p.legend(bbox_to_anchor=(-0.98, 2.7, 1.75, .1), loc=4,
                           ncol=4, mode="expand", borderaxespad=0.,fontsize=10,frameon=False)
        filename="InText-W{}-S{}.png".format(self.label,self.overall[0]["window_size"],step_size)
        # legenda no superior direito
        tname="{}{}".format(self.tdir,filename)
        p.savefig(tname)
        print("written image file:", tname)
        #p.show()
    def plotFirstPage(self):
        p.clf()
        fig = matplotlib.pyplot.gcf()
        fig.set_size_inches(7.,8.4)
        step_size=self.overall[0]["step_size"]
        ate=self.overall[1][0].n_messages-self.overall[0]["window_size"]
        p.suptitle("Primary divisions. Window: %i messages.\nPlacement resolution: %i messages. %s" % (self.overall[0]["window_size"],step_size,self.label))
        self.plotFracs("degree","521",ate,step_size)
        self.plotFracs("strength","522",ate,step_size)
        self.plotFracs("in-degree","523",ate,step_size)
        self.plotFracs("in-strength","524",ate,step_size)
        self.plotFracs("out-degree","525",ate,step_size)
        self.plotFracs("out-strength","526",ate,step_size)
        self.plotMeasure("total weight","527",ate,step_size)
        self.plotMeasure("number of edges","528",ate,step_size)
        self.plotMeasure("number of vertices","529",ate,step_size)
        self.plotMeasure("center, periphery and discon.","5,2,10",ate,step_size)
        p.subplots_adjust(left=0.105,bottom=0.05,right=0.98,top=0.9,wspace=0.28,hspace=0.69)
        filename="{}-W{}-S{}.png".format(self.label,self.overall[0]["window_size"],step_size)
        p.savefig("{}/{}".format(self.tdir,filename))
        # simple plot:
        p.clf()
        fig = matplotlib.pyplot.gcf()
        fig.set_size_inches(7.,5.4)
        p.suptitle("Compound divisions. Window: %i messages.\nPlacement resolution: %i messages. %s" % (self.overall[0]["window_size"],step_size,self.label))
        self.plotFracs("exclusivist","321",ate,step_size)
        self.plotFracs("inclusivist","322",ate,step_size)
        self.plotFracs("exclusivist cascade","323",ate,step_size)
        self.plotFracs("inclusivist cascade","324",ate,step_size)
        self.plotFracs("exclusivist externals","325",ate,step_size)
        self.plotFracs("inclusivist externals","326",ate,step_size)
        p.subplots_adjust(left=0.09,bottom=0.08,right=0.99,top=0.86,wspace=0.28,hspace=0.55)
        filename="{}-W{}-S{}_.png".format(self.label,self.overall[0]["window_size"],step_size)
        p.savefig("{}/{}".format(self.tdir,filename))

    def drawTimelines(self):
        self.plotFirstPage()

    def fractionLengths(self,list_of_lists,total):
        llist=[len(i) for i in list_of_lists]
        return [i/total for i in llist]
